#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 2023/07/01
@author: juliengautier & leopold Rousseau
window image
"""

from PyQt6.QtWidgets import QApplication, QVBoxLayout, QWidget
from PyQt6.QtWidgets import QLabel, QMainWindow, QFileDialog,QStatusBar
from PyQt6 import QtCore, QtGui
from PyQt6.QtCore import Qt
from PyQt6.QtGui import QAction
from PyQt6.QtGui import QIcon, QShortcut
from PIL import Image
import sys
import time
import pyqtgraph as pg  # pyqtgraph biblio permettent l'affichage
import numpy as np
import qdarkstyle  # pip install qdakstyle https://github.com/ColinDuquesnoy/QDarkStyleSheet  sur conda
import os
import pathlib
from scipy.ndimage import median_filter
#from winCrop import WINCROP
from visu.WinCut import GRAPHCUT
from visu.winMeas import MEAS
from visu.InputElectrons import InputE
from visu.CalculTraj import WINTRAJECTOIRE

class WINSPECTRO(QMainWindow):
    signalMeas = QtCore.pyqtSignal(object)
    def __init__(self, parent=None, file=None, conf=None, name='VISU',**kwds):
        
        super().__init__()
        self.name = name
        self.parent = parent
        p = pathlib.Path(__file__)
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else:
            self.conf = conf
        self.confSpectro = QtCore.QSettings(str(p.parent / 'default_values.ini'), QtCore.QSettings.Format.IniFormat)
        sepa = os.sep
        self.icon = str(p.parent) + sepa+'icons' + sepa
        self.path = self.conf.value(self.name+"/path")
        self.isWinOpen = False
        self.setWindowTitle('Electrons spectrometer')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.left = 100
        self.top = 30
        self.width = 800
        self.height = 800
        self.setGeometry(self.left, self.top, self.width, self.height)
        
        self.nomFichier = ''
        self.ite = None
        if "confMot" in kwds:
            self.confMot = kwds["confMot"]  # le Qsetting des moteurs
            self.winM = MEAS(parent=self, confMot=self.confMot, conf=self.conf, name=self.name)
        else:
            self.winM = MEAS(parent=self, conf=self.conf, name=self.name)
        self.winM .setWindowTitle('SPECTRO MEASUREMENTS')
        self.setWindowIcon(QIcon('./icons/LOA.png'))
        self.winInputE=InputE(parent=self)
        self.winTrajectory = WINTRAJECTOIRE(parent=self)
        # self.win2=WINCROP()
        # self.win3=WINCROP()
        self.winplot=GRAPHCUT()
        self.setup()
        self.shortcut()
        self.actionButton()

        if file is not None:
            self.dataOrg = Image.open(file)
            self.dataOrg = np.array(self.dataOrg)
            self.dataOrg = np.rot90(self.dataOrg, 3)
        
            self.SpectroChanged()
        else :
            self.spectroInputValue()

    def setup(self):
        
        self.toolBar = self.addToolBar('tools')
        self.toolBar.setMovable(False)

        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        self.statusBar = QStatusBar()
        self.setContentsMargins(0, 0, 0, 0)
        
        self.setStatusBar(self.statusBar)
        self.fileMenu = menubar.addMenu('&File')
        self.ImageMenu = menubar.addMenu('&Image')
        self.optionMenu = menubar.addMenu('&Options')
        self.AnalyseMenu = menubar.addMenu('&Analyse')
        self.openAct = QAction(QtGui.QIcon(self.icon+"Open.png"), 'Open File', self)
        self.openAct.setShortcut('Ctrl+o')
        self.toolBar.addAction(self.openAct)
        self.openAct.triggered.connect(self.OpenF)
        
        self.fileMenu.addAction(self.openAct)
        
        self.saveAct = QAction(QtGui.QIcon(self.icon+"disketteSave.png"), 'Save file', self)
        self.saveAct.setShortcut('Ctrl+s')
        self.toolBar.addAction(self.saveAct)
        self.saveAct.triggered.connect(self.SaveF)
        
        self.fileMenu.addAction(self.saveAct)

        self.InputEAct=QAction(QtGui.QIcon(self.icon+"pref.png"),
                                     'Input Electrons', self)
        self.InputEAct.triggered.connect(
                lambda: self.open_widget(self.winInputE))
        self.fileMenu.addAction(self.InputEAct)

        self.CalibEAct=QAction(QtGui.QIcon(self.icon+"pref.png"),
                                     'Trajectory', self)
        self.CalibEAct.triggered.connect(
                lambda: self.open_widget(self.winTrajectory))
        self.fileMenu.addAction(self.CalibEAct)


        self.checkBoxScale = QAction(QtGui.QIcon(self.icon+"expand.png"), ' Auto Scale on', self)
        self.checkBoxScale.setCheckable(True)
        self.checkBoxScale.setChecked(True)
        self.toolBar.addAction(self.checkBoxScale)
        self.ImageMenu.addAction(self.checkBoxScale)
        self.checkBoxScale.triggered.connect(self.checkBoxScaleImage)
        self.MeasButton = QAction(QtGui.QIcon(self.icon+"laptop.png"),
                                      'Measure', self)
        self.MeasButton.setShortcut('ctrl+m')
        self.MeasButton.triggered.connect(self.Measurement)
        self.AnalyseMenu.addAction(self.MeasButton)
        
        self.rectangleButton = QAction(QtGui.QIcon(self.icon+"rectangle.png"),
                                       'Add Rectangle', self)
        self.toolBar.addAction(self.rectangleButton)
        self.rectangleButton.triggered.connect(self.Rectangle)
        # rectangle for measurement
        self.plotRect = pg.RectROI([0, 0], [50,50],
                                   pen='g')
        self.plotRect.sigRegionChangeFinished.connect(self.RectChanged)

        self.label_CrossValue = QLabel()
        self.label_CrossValue.setStyleSheet("font:13pt")
        
        self.label_Cross = QLabel()
        # self.label_Cross.setMaximumHeight(20)
        self.label_Cross.setMaximumWidth(210)
        self.label_Cross.setStyleSheet("font:12pt")
        self.statusBar.addPermanentWidget(self.label_Cross)
        self.statusBar.addPermanentWidget(self.label_CrossValue)

        self.checkBoxBg = QAction('Background Substraction On', self)
        self.checkBoxBg.setCheckable(True)
        self.checkBoxBg.setChecked(False)
        self.checkBoxBg.triggered.connect(self.SpectroChanged)
        self.ImageMenu.addAction(self.checkBoxBg)

        self.vbox2 = QVBoxLayout()
        self.winImage = pg.GraphicsLayoutWidget()
        self.winImage.setContentsMargins(0, 0, 0, 0)
        self.winImage.setAspectLocked(False)
        # self.winImage.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        self.winImage.ci.setContentsMargins(0, 0, 0, 0)
        self.vbox2.addWidget(self.winImage)
        self.vbox2.setContentsMargins(0, 0, 0, 0)
        
        self.p1 = self.winImage.addPlot()
        self.imh = pg.ImageItem()
        self.axeX = self.p1.getAxis('bottom')
        self.axeY = self.p1.getAxis('left')
        self.p1.addItem(self.imh)
        self.p1.setMouseEnabled(x=False, y=False)
        self.p1.setContentsMargins(0, 0, 0, 10)
    
        #self.p1.setAspectLocked(True, ratio=1)
        self.p1.showAxis('right', show=False)
        self.p1.showAxis('top', show=False)
        self.p1.showAxis('left', show=True)
    
        self.axeX = self.p1.getAxis('bottom')
        self.axeY = self.p1.getAxis('left')
        self.axeX.setLabel(' Energy (Mev) ')
        self.axeY.setLabel( ' mrad ')

        self.winImage2 = pg.GraphicsLayoutWidget()
        self.winPLOT = self.winImage2.addPlot()
        self.vbox2.addWidget(self.winImage2)
        self.pCut = self.winPLOT.plot()
        self.winPLOT.setLabel('bottom', 'Energy (Mev)')
        self.winPLOT.showAxis('right', show=False)
        self.winPLOT.showAxis('top', show=False)
        self.winPLOT.showAxis('left', show=True)
        self.winPLOT.showAxis('bottom', show=True)
        self.axeX = self.winPLOT.getAxis('bottom')
        self.p1.setAxis=self.axeX

        MainWidget = QWidget()
        MainWidget.setLayout(self.vbox2)
        self.setCentralWidget(MainWidget)

        # histogramvalue()
        self.hist = pg.HistogramLUTItem()
        self.hist.setImageItem(self.imh)
        self.hist.autoHistogramRange()
        self.hist.gradient.loadPreset('flame')
        # rectangle Selection ROI spectro
        self.rectSelectSpectro = pg.RectROI([50, 50], [4*10, 20],
                                            pen='w')
        self.rectSelectSpectro.setPos([self.winInputE.wmin.value(),self.winInputE.hmin.value()])
        self.rectSelectSpectro.setSize([self.winInputE.wmax.value() - self.winInputE.wmin.value(),
                                        self.winInputE.hmax.value() - self.winInputE.hmin.value()])

        if self.parent is not None:
            # if signal emit in another thread (see visual)
            self.parent.signalSpectro.connect(self.Display)

    def checkBoxScaleImage(self):

        if self.checkBoxScale.isChecked():
            self.checkBoxScale.setIcon(QtGui.QIcon(self.icon+"expand.png"))
            self.checkBoxScale.setText('Auto Scale On')
        else:
            self.checkBoxScale.setIcon(QtGui.QIcon(self.icon+"minimize.png"))
            self.checkBoxScale.setText('Auto Scale Off')
    
    def Display(self, data):

        # self.open_widget(self.win2)
        # self.open_widget(self.winplot)
        # self.open_widget(self.win3)
        # self.win2.setWindowTitle('win Background crop )')
        
        self.dataOrg=data
        self.data = data[self.wmin:self.wmax+1,self.hmin:self.hmax+1]
        self.dimx = self.data.shape[0]
        self.dimy = self.data.shape[1]
        
        if self.checkBoxBg.isChecked() is True:
            # print('hmin hmax',self.winInputE.hminBg,self.winInputE.hmaxBg)
            #self.bg = self.winInputE.rectSelectBack.getArrayRegion(spectrum_2D.T, self.imh)
            self.bg =self.data[:, self.winInputE.hminBg:self.winInputE.hmaxBg ] # 

            # self.win3.Display(self.bg)
            # self.win3.setWindowTitle('win bg crop')
            if self.filterMed >0:
                self.data = np.array(median_filter(self.data , self.filterMed))
                self.bg = median_filter(self.bg , self.filterMed)
            self.bg = self.bg.mean(axis=1)
            
            # print('shape bg ligne',self.bg.shape)
            # print('data size' ,self.data.shape)
            # self.winplot.PLOT(self.bg)
            
            self.bg2 = np.tile(self.bg,self.data.shape[1])
            # print('shape bg tile',self.bg.shape)
            #self.bg = np.reshape(self.bg2,[self.data.shape[1],self.data.shape[0]])
            self.bg = np.reshape(self.bg2,[self.data.shape[0],self.data.shape[1]],order='F')
            # print('data shape' ,self.data.shape)
            # print('bg size 2' ,self.bg.shape)
            # self.win2.Display(self.bg)
            # self.win2.setWindowTitle('win bg reconstitué )')
            self.data=self.data - self.bg
        else:
            if self.filterMed >0:
                self.data = np.array(median_filter(self.data , self.filterMed))

        self.data[self.data<0] = 0 #change all negative values to 0
        # to be changed no need to be interpolate at each display
        #self.energy_max = int(np.interp((self.s0-self.dimx+1)/self.ppmm, self.slist[::-1], self.elist[::-1]))
        self.energy_max=(np.interp((self.wmax-self.s0)/self.ppmm, self.slist, self.elist))
        self.energy_min =(np.interp((self.wmin-self.s0)/self.ppmm, self.slist,self.elist))
        print('Emin:', self.energy_min,'Emax:', self.energy_max,self.ppmm)
        print('Xmin', (self.wmin-self.s0)/self.ppmm,'Xmax', (self.wmax-self.s0)/self.ppmm)
        self.E = np.linspace(self.energy_min,self.energy_max,self.npoints) 
        ds_dE = abs(np.interp(self.E, self.elist, self.dsdelist))
        
        #  Calculate angle range:
        
        p_y0 = np.unravel_index(np.argmax(self.data), self.data.shape)[1]
        self.theta_max = (self.dimy-1 - p_y0) / self.ppmm / self.ssd  # minimum angle [mrad] 
        self.theta_min = (- p_y0) / self.ppmm / self.ssd  # maximum angle [mrad] 
        deltaTheta = self.theta_max - self.theta_min
        self.scaleY = deltaTheta/self.dimy
        self.scaleX = (self.energy_max-self.energy_min)/self.npoints

        #  Create and fill a new array for the 2D spectrum
        spectrum_2D = np.empty((self.dimy, 0))
        
        for i in range(self.npoints-1):
            # self.s0 position pixel 0 lanex
            # position in px of E in the filtered image = mm/0 lanex * ppm +px_zerolanex -pixel fenetre
            px_float = round(self.ppmm * np.interp(self.E[i], self.elist, self.slist)) - self.wmin + self.s0
            #p rint('pixel dans data win',px_float,self.E[i],round(self.ppmm * np.interp(self.E[i], self.elist, self.slist)))
            # print('pixel dans data org',self.ppmm*np.interp(self.E[i], self.elist, self.slist),self.E[i])
            px0 = int(np.trunc( px_float)) #floor
            # print(i,px0,self.data.shape)
            px1 = px0 +1
            if px1 > self.data.shape[0]-1 :
                px1 = px1 - 1 
            row = np.array([(px1-px_float) * self.data[px0,:] + (px_float-px0) * self.data[px1,:]]).T
            spectrum_2D = np.hstack([ spectrum_2D , row * ds_dE[i] ]) 
                
        spectrum_2D = np.hstack([ spectrum_2D , np.array([self.data[-1,:]]).T* ds_dE[i] ])
        self.theta=np.linspace(self.theta_min,self.theta_max,self.data.shape[1])
    
            #  Translation and scaling the image
        tr = QtGui.QTransform()  # prepare ImageItem transformation:
        tr.scale(self.scaleX, self.scaleY)       # scale horizontal and vertical axes
        self.transX=self.energy_min/self.scaleX
        self.transY=-p_y0
        tr.translate(self.energy_min/self.scaleX, self.transY) #  to locate maximum at axis origin
        self.imh.setTransform(tr) # assign transform

        if self.checkBoxScale.isChecked():
            self.imh.setImage(spectrum_2D.T, autoLevels=True, autoDownsample=True) 
        else:
            self.imh.setImage(spectrum_2D.T, autoLevels=False, autoDownsample=True) 

        self.p1.setXRange(self.energy_min,self.energy_max)
        self.pCut.setData(self.E,spectrum_2D.sum(axis=0))
        self.winPLOT.setXRange(self.energy_min,self.energy_max)
        self.data = spectrum_2D.T
        self.dimx = self.data.shape[0]
        self.dimy = self.data.shape[1]

        if self.winM.isWinOpen is True:  # measurement update
            if self.ite == 'rect': # rest selection is selected
                self.RectChanged()
                self.Measurement()
            else:
                self.Measurement()
        # except:
        #     self.errorDisplay = True
        #     print('error Display: Calibrate')


    def shortcut(self):
        # keyboard shortcut
        self.shortcutPu = QShortcut(QtGui.QKeySequence("+"), self)
        self.shortcutPu.activated.connect(self.paletteup)
        self.shortcutPu.setContext(Qt.ShortcutContext(3))
        # 3: The shortcut is active when its parent widget, or any of its children has focus. default O The shortcut is active when its parent widget has focus.
        self.shortcutPd = QtGui.QShortcut(QtGui.QKeySequence("-"), self)
        self.shortcutPd.activated.connect(self.palettedown)
        self.shortcutPd.setContext(Qt.ShortcutContext(3))
        # mousse action:
        self.proxy = pg.SignalProxy(self.p1.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        # self.p1.scene().sigMouseClicked.connect(self.mouseClick)
        # self.vb=self.p1.vb

    def actionButton (self):
        self.winInputE.selectButton.clicked.connect(self.selectDimSpectro)
        self.rectSelectSpectro.sigRegionChangeFinished.connect(self.changeDimSpectro)
        self.winInputE.wmin.editingFinished.connect(self.SpectroChanged)
        self.winInputE.wmax.editingFinished.connect(self.SpectroChanged)
        self.winInputE.hmin.editingFinished.connect(self.SpectroChanged)
        self.winInputE.hmax.editingFinished.connect(self.SpectroChanged)    
        self.winInputE.medfilt.editingFinished.connect(self.SpectroChanged)
        self.winInputE.npoints.editingFinished.connect(self.SpectroChanged)
        self.winInputE.ppmm.editingFinished.connect(self.SpectroChanged)
        self.winInputE.pps0.editingFinished.connect(self.SpectroChanged)
        self.winInputE.ssd.editingFinished.connect(self.SpectroChanged)
        self.winInputE.count.editingFinished.connect(self.SpectroChanged)
        self.winInputE.selected_dsde_label.textChanged.connect(self.SpectroChanged)

    def paletteup(self):
        # change the color scale
        levels = self.imh.getLevels()
        if levels[0] is None:
            xmax = self.data.max()
            xmin = self.data.min()
        else:
            xmax = levels[1]
            xmin = levels[0]
        self.imh.setLevels([xmin, xmax-(xmax - xmin) / 10])
        self.hist.setHistogramRange(xmin, xmax)

    def palettedown(self):
        levels = self.imh.getLevels()
        if levels[0] is None:
            xmax = self.data.max()
            xmin = self.data.min()
        else:
            xmax = levels[1]
            xmin = levels[0]
        self.imh.setLevels([xmin, xmax + (xmax - xmin) / 10])
        self.hist.setHistogramRange(xmin, xmax)

    def OpenF(self, fileOpen=False):

        fname = QFileDialog.getOpenFileNames(self, "Open File")
        self.openedFiles = fname[0]
        fichier = self.openedFiles[0]
        ext = os.path.splitext(fichier)[1]
        
        if ext == '.txt':  # text file
            data = np.loadtxt(str(fichier))
        elif ext == '.TIFF' or ext == '.tif' or ext == '.Tiff' or ext == '.jpg' or ext == '.JPEG' or ext == '.png':  # tiff File
            dat = Image.open(fichier)
            data = np.array(dat)
            data = np.rot90(data, 3)
        
        self.Display(data)
        

    def SaveF(self):
        # save as tiff
        fname = QFileDialog.getSaveFileName(self, "Save data as TIFF", self.path)
        self.path = os.path.dirname(str(fname[0]))
        fichier = fname[0]
        print(fichier, ' is saved')
        self.conf.setValue(self.name+"/path", self.path)
        time.sleep(0.1)
        self.dataS = np.rot90(self.data, 1)
        img_PIL = Image.fromarray(self.dataS)
        img_PIL.save(str(fname[0])+'.TIFF', format='TIFF')
        
    def mouseMoved(self, evt):
        pos = evt[0]  # using signal proxy turns original arguments into a tuple
        if self.p1.sceneBoundingRect().contains(pos):
            mousePoint = self.p1.vb.mapSceneToView(pos)
            self.xMouse = (mousePoint.x())
            self.yMouse = (mousePoint.y())
            if ((self.xMouse > self.energy_min and self.xMouse < self.energy_max ) and (self.yMouse > self.theta_min and self.yMouse < self.theta_max)):
                self.xc = self.xMouse
                self.yc = self.yMouse
                try:
                    dataCross = (self.data[int(self.xc), int(self.yc)] ) * self.countPerPixel
                except:
                    dataCross = 0  # evoid to have an error if cross if out of the image
                    self.xc = 0
                    self.yc = 0
                
                self.label_Cross.setText('x=' + str(round(self.xc,1)) + ' Mev  '+ ' y=' + str(round(self.yc,1)) + ' mrad  ' )
                
                dataCross = round(dataCross, 3)  # take data  value  on the cross
                self.label_CrossValue.setText(' v.=' + str(dataCross)+ ' C/Mev')

    def Rectangle(self):
        if self.ite == 'rect':
            self.p1.removeItem(self.plotRect)
            self.ite = None
        else:
            self.ite = 'rect'
            self.p1.addItem(self.plotRect)
            self.plotRect.setPos([100, 0])

    def RectChanged(self):
        '''Take ROIself.s0 = self.winInputE.pps0.value() # pixel position of high energy
        self.wmin = int(self.winInputE.wmin.value()) # dimension
        self.wmax = int(self.winInputE.wmax.value())
        self.hmin = int(self.winInputE.hmin.value())
        self.hmax = int(self.winInputE.hmax.value())
        self.ppmm = self.winInputE.ppmm.value() # pixel per millimeter
        self.npoints = int(self.winInputE.npoints.value()) # nombre de point
        self.ssd= self.winInputE.ssd.value()  # source screen distance
        self.slist = self.winInputE.slist
        self.elist = self.winInputE.elist
        self.dsdelist = self.winInputE.dsdelist
        self.filterMed = int(self.winInputE.medfilt.value())
        self.countPerPixel = self.winInputE.count.value()
        '''
        self.cut = (self.plotRect.getArrayRegion(self.data, self.imh))
        self.xini=self.plotRect.pos()[0]
        self.yini=self.plotRect.pos()[1]
        
        # # Rectangle stay inside view
        # if self.plotRect.pos()[0] < 0:
        #     self.plotRect.setPos([0, self.plotRect.pos()[1]])

        # if self.plotRect.pos()[0]+self.plotRect.size()[0] > self.dimx:
        #     x = self.plotRect.pos()[0]
        #     y = self.plotRect.pos()[1]
        #     sizex = self.dimx-self.plotRect.pos()[0]
        #     sizey = self.plotRect.size()[1]
        #     self.plotRect.setSize([sizex, sizey], update=False)
        #     self.plotRect.setPos([x, y])

        # if self.plotRect.pos()[1]+self.plotRect.size()[1] > self.dimy:
        #     x = self.plotRect.pos()[0]
        #     y = self.plotRect.pos()[1]
        #     sizex = self.plotRect.size()[0]
        #     sizey = self.dimy-self.plotRect.pos()[1]
        #     self.plotRect.setSize([sizex, sizey], update=False)
        #     self.plotRect.setPos([x, y])

        # if self.plotRect.pos()[1] < 0:
        #     self.plotRect.setPos([self.plotRect.pos()[0], 0])

    def spectroFunct(self):
        self.open_widget(self.winSpectro)
        self.SpectroChanged()

    def spectroInputValue(self):
        self.s0 = self.winInputE.pps0.value() # pixel position of lanex origin
        self.wmin = int(self.winInputE.wmin.value()) # dimension
        self.wmax = int(self.winInputE.wmax.value())
        self.hmin = int(self.winInputE.hmin.value())
        self.hmax = int(self.winInputE.hmax.value())
        self.ppmm = self.winInputE.ppmm.value() # pixel per millimeter
        self.npoints = int(self.winInputE.npoints.value()) # nombre de point
        self.ssd= self.winInputE.ssd.value()  # source screen distance
        self.slist = self.winInputE.slist
        self.elist = self.winInputE.elist
        self.dsdelist = self.winInputE.dsdelist
        self.filterMed = int(self.winInputE.medfilt.value())
        self.countPerPixel = self.winInputE.count.value()

    def SpectroChanged(self):
        # print('specto changeged')
        self.spectroInputValue()
        #self.signalSpectro.emit(self.data)
        self.Display(self.dataOrg)
    
    def selectDimSpectro(self):
        
        if self.winInputE.buttonSelected is False :
            self.rectSelectSpectro.setPos([self.winInputE.wmin.value(),self.winInputE.hmin.value()])

            self.rectSelectSpectro.setSize([self.winInputE.wmax.value() - self.winInputE.wmin.value(),
                                        self.winInputE.hmax.value() - self.winInputE.hmin.value()])
            if self.parent is not None:
                self.parent.p1.addItem(self.rectSelectSpectro)
                self.winInputE.buttonSelected = True
        else:
            self.winInputE.buttonSelected = False
            if self.parent is not None:
                self.parent.p1.removeItem(self.rectSelectSpectro)
        
    
    def changeDimSpectro(self):
        self.winInputE.wmin.setValue(int(self.rectSelectSpectro.pos()[0]))
        self.winInputE.wmax.setValue(int(self.rectSelectSpectro.pos()[0] + self.rectSelectSpectro.size()[0] ))
        self.winInputE.hmin.setValue(int(self.rectSelectSpectro.pos()[1]))
        self.winInputE.hmax.setValue(int(self.rectSelectSpectro.pos()[1] + self.rectSelectSpectro.size()[1] ))
        self.SpectroChanged()

    def Measurement(self):
        '''widget for measurement on all image or ROI  (max, min mean ...)
        '''
        if self.ite == 'rect':
            self.RectChanged()
            self.winM.setFile(self.nomFichier)
            self.open_widget(self.winM)
            MeasData=[self.cut*self.countPerPixel,self.xini+self.transX,self.yini+self.transY,self.scaleX,self.scaleY]
            self.signalMeas.emit(MeasData)
      
        if self.ite is None:
            self.winM.setFile(self.nomFichier)
            self.open_widget(self.winM)
            MeasData=[self.data* self.countPerPixel,self.transX,self.transY, self.scaleX, self.scaleY]
            self.signalMeas.emit(MeasData)

    def open_widget(self, fene):
        """ open new widget
        """
        
        if fene.isWinOpen is False:
            fene.setup
            fene.isWinOpen = True
            fene.show()
        else:
            # fene.activateWindow()
            # fene.raise_()
            fene.showNormal()

    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        
        if self.winInputE.isWinOpen is True:
            self.winInputE.close()
        if self.winM.isWinOpen is True:
            self.winM.close()
        time.sleep(0.1)

        event.accept()
    
        
if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    file= str(pathlib.Path(__file__).parents[0])+'/tir_025.TIFF'
    e =WINSPECTRO(name='VISU', file=file)
    e.show()
    appli.exec_()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb  8 14:46:20 2019
Windows for plot
@author: juliengautier
"""

import pyqtgraph as pg  # pyqtgraph biblio permettent l'affichage 

import qdarkstyle  # pip install qdakstyle https://github.com/ColinDuquesnoy/QDarkStyleSheet  sur conda
from PyQt6.QtWidgets import QApplication, QVBoxLayout, QMainWindow
from PyQt6.QtGui import QIcon
from PyQt6 import QtCore
import os
import sys
import time
from PyQt6.QtCore import pyqtSlot
import numpy as np
import pathlib
import pyqtgraph.opengl as gl


class GRAPH3D(QMainWindow):

    def __init__(self, symbol=True, title='Plot3D', conf=None, name='VISU'):
        super(GRAPH3D, self).__init__()
        p = pathlib.Path(__file__)
        sepa = os.sep
        self.title = title
        self.icon = str(p.parent) + sepa + 'icons' + sepa
        self.left = 100
        self.top = 30
        self.width = 800
        self.height = 800
        self.setGeometry(self.left, self.top, self.width, self.height)
        self.isWinOpen = False
        self.symbol = symbol
        self.name = name
        self.path = None
        self.axisOn = False
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.IniFormat)
        else:
            self.conf = conf

        self.setup()
        self.setWindowTitle(self.title)
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
        
    def setup(self):
        vLayout = QVBoxLayout()

        #  Add a grid to the view
        self.w = gl.GLViewWidget()
        # g = gl.GLGridItem()
        # g.scale(20,20,1)
        # g.setDepthValue(220)  # draw grid after surfaces since they may be translucent
        # self.w.addItem(g)
        vLayout.addWidget(self.w)
        self.w.setCameraPosition(distance=1000)
        self.setCentralWidget(self.w)
        # self.w.show()
        
    def Plot3D(self, data, axisX=None, axisY=None, symbol=True, pen=True, label=None):
        print('ici plot 3d')
        self.data = data
        self.dimx = np.shape(self.data)[0]
        self.dimy = np.shape(self.data)[1]
        self.pen = pen
        self.axisX = axisX
        self.axisY = axisY
        
        if self.axisX is None or self.axisY is None:
            
            z = self.data  # [:self.dimx,:self.dimy]
            zmax = z.max()
            # zmin = z.min()
            if zmax == 0:
                zmax = 1
            z = z/((zmax))
            
            p1 = gl.GLSurfacePlotItem(z=z, shader='heightColor', computeNormals=False, smooth=False)  # .reshape(50*50,4))
            # p1.shader()['colorMap'] = np.array([0.2, 2, 0.5, 0.2, 1, 1, 0.2, 0, 2])
            p1.scale(1, 1, zmax)
              
            self.w.addItem(p1)
        else:
            print('')
            # create 3D array to pass it to plotting function
# 		vetrs3D = np.zeros([nbv,3])
# 		if vertexes.shape == (nbv,3): vetrs3D = vertexes
# 		else: 
# 			print 'Converting 2D to 3D'
# 			vetrs3D[:,:2] = vertexes
# 		# compute distance to set initial camera pozition
# 		dst = max(vertexes[:,0].max() - vertexes[:,0].min(), vertexes[:,1].max() - vertexes[:,1].min())
# 		view.setCameraPosition(distance=dst)

# 		plt = gl.GLMeshItem(vertexes=vetrs3D, faces=faces, faceColors=colors, drawEdges=drawEdges, smooth=smooth)
            
        # p1.scale(16./49., 16./49., 1.0)
        # p1.translate(-18, 2, 0)
       
    def SetTITLE(self, title):
        self.setWindowTitle(title)
    
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        time.sleep(0.1)
        event.accept()


if __name__ == "__main__":
    z = pg.gaussianFilter(np.random.normal(size=(50, 50)), (1, 1))
    appli = QApplication(sys.argv) 
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = GRAPH3D()
    e.Plot3D(data=z)
    e.show()
    appli.exec_()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Licence.txt from original archive:
#   Copyright (c) 2006, Marcel Leutenegger
# 	All rights reserved.
#
# 	Redistribution and use in source and binary forms, with or without
# 	modification, are permitted provided that the following conditions are
# 	met:
#
# 	    * Redistributions of source code must retain the above copyright
# 	      notice, this list of conditions and the following disclaimer.
# 	    * Redistributions in binary form must reproduce the above copyright
# 	      notice, this list of conditions and the following disclaimer in
# 	      the documentation and/or other materials provided with the distribution
# 	    * Neither the name of the Ecole Polytechnique F�d�rale de Lausanne,
# 	      Laboratoire d'Optique Biom�dicale nor the names of its contributors may
# 	      be used to endorse or promote products derived from this software
# 	      without specific prior written permission.
#  	      
#	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# 	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# 	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOS
# 	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# 	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# 	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# 	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# 	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# 	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# 	POSSIBILITY OF SUCH DAMAGE.

# pysifreader: Read Andor SIF multi-channel image file.
#
# Date: 23nd October, 2009
# Version: 1.0.1
# History:
#  1.0   -- 22/10/09 -- Initial release
#  1.0.1 -- 23/10/09 -- Streamlined and made code more portable
#
# Usage:
#     [data, back, ref] = SIFread(filename)
#
# Synopsis:
#     Read the image data, background and reference from file.
#     Return the image data, background and reference in variables of type SIFimage.
#
# Module pysifreader methods:
#  SIFread(filename)       Reads the file and returns the image data
#
# SIFimage methods:
#  .printInfo()            Prints all the available information (see below)
#
# SIFimage Properties:
#  .temperature            CCD temperature [deg. C]
#  .exposureTime           Exposure time [s]
#  .cycleTime              Time per full image take [s]
#  .accumulateCycles       Number of accumulation cycles
#  .accumulateCycleTime    Time per accumulated image [s]
#  .stackCycleTime         Interval in image series [s]
#  .pixelReadoutTime       Time per pixel readout [s]
#  .gainDAC                DAC gain
#  .detectorType           CCD type
#  .detectorSize           Number of read CCD pixels [x,y]
#  .seqNum                 Sequence number (0=none, 1=data, 2=background, 3=reference)
#  .fileName               Original file name
#  .shutterTime            Time to open/close the shutter [s]
#  .frameAxis              Axis unit of CCD frame
#  .dataType               Type of image data
#  .imageAxis              Axis unit of image
#  .imageArea              Image limits [x1,y1,first image; x2,y2,last image]
#  .frameArea              Frame limits [x1,y1; x2,y2]
#  .frameBins              Binned pixels [x,y]
#  .timeStamp              Time stamp in image series
#  .imageData              Image data (x,y,t)

# Note from Marcel Leutenegger:
#
#  The file format was reverse engineered by identifying known
#  information within the corresponding file. There are still
#  non-identified regions left over but the current summary is
#  available on request.
#

# Original MATLAB code by: � Marcel Leutenegger, November 2008
#  http://www.mathworks.com/matlabcentral/fileexchange/11224
# Ported to python by Andr� Xuereb, October 2009
#  <andre.xuereb@soton.ac.uk>


class SIFimage:
    # 	def printInfo(self):
    # 		print ('Temperature: %d deg. C' % self.temperature)
 # 		print 'Exposure time: %f s' % self.exposureTime
	# 		print 'Time per full image take: %f s' % self.cycleTime
	# 		print 'Number of accumulation cycles: %d' % self.accumulateCycles
	#		print 'Time per accumulated image: %f s' % self.accumulateCycleTime
# 		print 'Interval in image series: %f s' % self.stackCycleTime
# 		print 'Time per pixel readout: %f s' % self.pixelReadoutTime
# 		print 'DAC gain: %f' % self.gainDAC
# 		print 'Time to open/close the shutter: %f s/%f s' % (self.shutterTime[0], self.shutterTime[1])
# 		print
# 		print 'CCD type: %s' % self.detectorType
# 		print 'Number of read CCD pixels (x, y): (%d, %d)' % (self.detectorSize[0], self.detectorSize[1])
# 		print
# 		print 'File name: %s' % self.fileName
# 		print 'Sequence number: %d' % self.seqNum
# 		print 'Axis unit of CCD frame: %s' % self.frameAxis
# 		print 'Type of image data: %s' % self.dataType
#		print 'Axis unit of image: %s' % self.imageAxis
#		print 'Time stamp in image series: %d' % self.timeStamp
#		print
#		print 'Binned pixels (x, y): (%d, %d)' % (self.frameBins[0], self.frameBins[1])
#		print 'Frame limits (x1, y1; x2, y2): (%d, %d; %d, %d)' % (self.frameArea[0][0], self.frameArea[0][1], self.frameArea[1][0], self.frameArea[1][1])
#		print 'Image limits (x1, y1, first image; x2, y2, last image): (%d, %d, %d; %d, %d, %d)' % (self.imageArea[0][0], self.imageArea[0][1], self.imageArea[0][2], self.imageArea[1][0], self.imageArea[1][1], self.imageArea[1][2])

	def __init__(self):
		from pylab import array as p_array
		self.temperature = 0
		self.exposureTime = 0.
		self.cycleTime = 0.
		self.accumulateCycleTime = 0.
		self.accumulateCycles = 0.
		self.stackCycleTime = 0.
		self.pixelReadoutTime = 0.
		self.gainDAC = 0.
		self.seqNum = 0
		self.shutterTime = [0., 0.]
		self.detectorType = ''
		self.detectorSize = [0, 0]
		self.fileName = ''
		self.frameAxis = ''
		self.dataType = ''
		self.imageAxis = ''
		self.imageArea = [[0, 0, 0], [0, 0, 0]]
		self.frameArea = [[0, 0], [0, 0]]
		self.frameBins = [0, 0]
		self.timeStamp = 0
		self.imageData = p_array([])

def sifread(file):
	from os import path, SEEK_CUR
	from pylab import array as p_array
	from sys import exit
	import array

	def readLine(file):
		t = ' '
		s = ''
		while len(t) > 0:
			t = file.read(1)
			if (len(t) >= 1) and (ord(t[0]) >= 32):
				s += t[0]
			else:
				t = ''
		return s

	def skipLine():
		if readLine(f) == '':
			f.close()
			error('Inconsistent image header.')
	def error(s):
		print('Error: %s' % s)
		exit()

	def readSection():
		def readString():
			s = readLine(f).strip()
			n = int(s)
			s = f.read(n)
			if ((n > 0) and (s == '')) or (n < 0):
				f.close()
				error('Inconsistent string.')
			return s

		def tofloat(o):
			for i in range(len(o)):
				o[i] = float(o[i])
			return o

		def toint(o):
			for i in range(len(o)):
				o[i] = int(o[i])
			return o

		def skipandread():
			s = ''
			while (s == ''):
				s = readLine(f).strip()
			return s

		image = SIFimage()
		o = skipandread().split()
		image.temperature = int(o[5])
		o = tofloat(skipandread().split())
		[image.exposureTime, image.cycleTime, image.accumulateCycleTime, image.accumulateCycles] = o[1:5]
		o = tofloat(skipandread().split())
		[image.stackCycleTime, image.pixelReadoutTime] = o[1:3]
		o = skipandread().split()
		image.gainDAC = float(o[2])
		o = skipandread().split()
		image.detectorType = o[0]
		o = toint(skipandread().split())
		image.detectorSize = o[0:2]
		o = skipandread().split()
		image.fileName = o[0]
		for n in range(3):
			o = skipandread()
		o = toint(skipandread().split())
		image.shutterTime = o[4:6]
		for n in range(15):
			o = skipandread()
		image.frameAxis = readString()
		image.dataType = readString()
		image.imageAxis = readString()
		o = skipandread().split()
		o += skipandread().split()
		o = toint(o)
		image.imageArea = [[o[1], o[4], o[6]], [o[3], o[2], o[5]]]
		image.frameArea=[[o[10], o[13]], [o[12], o[11]]]
		image.frameBins=[o[15], o[14]]
		s = int(((1 - (image.frameArea[0][0] - image.frameArea[1][0]))/(1.*image.frameBins[0]))*((1 - (image.frameArea[0][1] - image.frameArea[1][1]))/(1.*image.frameBins[1])))
		z = 1 + (image.imageArea[0][2] - image.imageArea[1][2])
		if (s != o[8]) or (o[8]*z != o[7]):
			f.close()
			error('Inconsistent image header.')
		for n in range(z):
			o = readString()
		o = array.array('i')
		o.fromfile(f, 1)
		image.timeStamp = o.tolist()[0]
		f.seek(-4, SEEK_CUR)
		o = array.array('f')
		o.fromfile(f, s*z)
		image.imageData = p_array(o.tolist()).reshape(image.frameArea[1][0], image.frameArea[1][1], z)
		readString()
		o = skipandread().split()
		next = int(o[0])
		return [image, next]

	if not path.exists(file):
		error('File (%s) does not exist.' % file)

	f = open(file, 'rb')
	if f < 0:
		error('Could not open the file.')
	if not (readLine(f) == 'Andor Technology Multi-Channel File'):
		f.close()
		error('Not an Andor SIF image file.')
	skipLine()
	[data,next] = readSection()
	data.seqNum = 1
	if (next == 1):
		[back,next] = readSection()
		back.seqNum = 2
		if (next == 1):
			ref = copy(back)
			ref.seqNum = 3
			back = readSection()
			back.seqNum = 2
		else:
			ref = SIFimage()
	else:
		back = SIFimage()
		ref = copy(back)
	f.close()

	return [data, back, ref]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb 13 16:02:40 2019

@author: juliengautier
"""

import qdarkstyle
from PyQt6.QtCore import pyqtSignal as Signal
from PyQt6 import QtCore, QtGui
from PyQt6.QtWidgets import QApplication, QCheckBox, QVBoxLayout, QHBoxLayout, QPushButton
from PyQt6.QtWidgets import QWidget, QLabel, QSpinBox, QLineEdit, QMessageBox, QFileDialog
from PyQt6.QtGui import QIcon
import sys
import os
import numpy as np
import socket as _socket
import pathlib
import time
import zmq
import uuid

class OPTION(QWidget):
    
    closeEventVar = QtCore.pyqtSignal(bool)
    emitChangeScale = QtCore.pyqtSignal(bool)
    emitChangeRot = QtCore.pyqtSignal(bool)

    def __init__(self, conf=None, name='VISU',parent=None):
        
        super().__init__()
        
        p = pathlib.Path(__file__)

        self.parent = parent
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else:
            self.conf = conf

        self.name = name
        sepa = os.sep
        self.icon = str(p.parent) + sepa+'icons' + sepa
        self.isWinOpen = False
        self.auto = False
        self.setWindowTitle('Options Auto Save & visualisation')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        
        # self.stepX=float(self.conf.value(self.name+"/stepX"))
        # self.stepY=float(self.conf.value(self.name+"/stepY"))

        self.shoot = int(self.conf.value(self.name+"/tirNumber"))
        self.setup()
        self.pathAutoSave = self.conf.value(self.name+"/path")
        self.pathBg = self.conf.value(self.name+"/pathBg")
        self.actionButton()
        self.dataBgExist = False
        self.rotateValue = 0
        self.modeTrig = False

    def setFile(self, file):
        self.nomFichier = file

    def setup(self):
        
        
        TogOff = self.icon+'Toggle_Off.png'
        TogOn = self.icon+'Toggle_On.png'
        TogOff = pathlib.Path(TogOff)
        TogOff = pathlib.PurePosixPath(TogOff)
        TogOn = pathlib.Path(TogOn)
        TogOn = pathlib.PurePosixPath(TogOn)
        self.setStyleSheet("QCheckBox::indicator{width: 30px;height: 30px;}""QCheckBox::indicator:unchecked { image : url(%s);}""QCheckBox::indicator:checked { image:  url(%s);}""QCheckBox{font :10pt;}" % (TogOff, TogOn))
        
        vbox1 = QVBoxLayout()
        self.buttonPath = QPushButton('Path : ')
        self.pathBox = QLineEdit(str(self.conf.value(self.name+"/pathAutoSave")))
        self.pathBox.setMaximumHeight(60)
        hbox1 = QHBoxLayout()
        hbox1.addWidget(self.buttonPath)
        hbox1.addWidget(self.pathBox)
        vbox1.addLayout(hbox1)
        
        hbox2 = QHBoxLayout()
        labelName = QLabel('Name : ')
        self.nameBox = QLineEdit(str(self.conf.value(self.name+"/nameFile")))
        self.nameBox.setMaximumHeight(30)
        hbox2.addWidget(labelName)
        hbox2.addWidget(self.nameBox)
        vbox1.addLayout(hbox2)
        
        hbox3 = QHBoxLayout()
        self.checkBoxTiff = QCheckBox('Save as TIFF', self)
        self.checkBoxTiff.setChecked(True)
        hbox2.addWidget(self.checkBoxTiff)
        
        self.checkBoxDate = QCheckBox('add date', self)
        self.checkBoxDate.setChecked(False)
        hbox2.addWidget(self.checkBoxDate)
        vbox1.addLayout(hbox3)
        
        hbox4 = QHBoxLayout()
        labelTirNumber = QLabel('Next number : ')
        self.checkBoxServer = QCheckBox('Auto', self)
        self.checkBoxServer.setChecked(False)
        hbox4.addWidget(self.checkBoxServer)
        
        self.tirNumberBox = QSpinBox()
        self.tirNumberBox.setMaximum(100000)
        self.tirNumberBox.setValue(self.shoot)
        hbox4.addWidget(labelTirNumber)
        hbox4.addWidget(self.tirNumberBox)
        vbox1.addLayout(hbox4)
        
        hbox5 = QHBoxLayout()
        self.buttonFileBg = QPushButton('Select Background File : ')
        self.fileBgBox = QLineEdit('bgfile not selected')
        self.fileBgBox.setMaximumHeight(60)
        hbox5 = QHBoxLayout()
        hbox5.addWidget(self.buttonFileBg)
        hbox5.addWidget(self.fileBgBox)
        vbox1.addLayout(hbox5)
        
        hMainLayout = QHBoxLayout()
        hMainLayout.addLayout(vbox1)
        self.setLayout(hMainLayout)
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        
    def actionButton(self):
        self.buttonPath.clicked.connect(self.PathChanged)
        self.pathBox.textChanged.connect(self.pathTextChanged)
        self.nameBox.textChanged.connect(self.nameFileChanged)
        self.tirNumberBox.valueChanged.connect(self.TirNumberChange)
        self.buttonFileBg.clicked.connect(self.selectBg)
        # self.fileBgBox.textChanged.connect(self.bgTextChanged)
        self.checkBoxServer.stateChanged.connect(self.checkBoxServerChange)

    def pathTextChanged(self):
        self.pathAutoSave = self.pathBox.text()
        self.conf.setValue(self.name+"/pathAutoSave", str(self.pathAutoSave))
        
    def PathChanged(self):
       
        self.pathAutoSave = str(QFileDialog.getExistingDirectory(self, "Select Directory", self.pathAutoSave))
        self.pathBox.setText(self.pathAutoSave)
        self.conf.setValue(self.name+"/pathAutoSave", str(self.pathAutoSave))
        
    def nameFileChanged(self):
        self.fileName = self.nameBox.text()
        self.conf.setValue(self.name+"/nameFile", str(self.fileName))
            
    def TirNumberChange(self):
        self.tirNumber = self.tirNumberBox.value()
        self.conf.setValue(self.name+"/tirNumber", self.tirNumber)
        self.conf.sync()
        
    def setTirNumber(self, tirNumber):
        self.tirNumber = tirNumber
        self.tirNumberBox.setValue(self.tirNumber)
 
    def selectBg(self):
        
        fname = QFileDialog.getOpenFileName(self, "Select a background file", self.pathBg, "Images (*.txt *.spe *.TIFF *.sif);;Text File(*.txt);;Ropper File (*.SPE);;Andor File(*.sif);; TIFF file(*.TIFF)")
        fichier = fname[0]
        ext = os.path.splitext(fichier)[1]
        self.fileBgBox.setText(fichier)
        
        self.conf.setValue(self.name+"/pathBg", os.path.dirname(fichier))
        
        self.dataBgExist = True
        self.parent.checkBoxBg.setChecked(True)
        self.parent.BackgroundF()

        if ext == '.txt':  # text file
            self.dataBg = np.loadtxt(str(fichier))
            self.dataBg = np.rot90(self.dataBg, 3)
        elif ext == '.spe' or ext == '.SPE':  # SPE file
            from winspec import SpeFile
            dataSPE = SpeFile(fichier)
            data1 = dataSPE.data[0]  # .transpose() # first frame
            self.dataBg = data1  # np.flipud(data1)
        elif ext == '.TIFF':  # tiff File
            from PIL import Image
            dat = Image.open(fichier)
            dat = np.rot90(dat, 3)
            self.dataBg = np.array(dat)
        elif ext == '.sif':
            from andor import SifFile
            sifop = SifFile()
            im = sifop.openA(fichier)
            self.dataBg = np.rot90(im, 3)
            
        else:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Critical)
            msg.setText("Wrong file format !")
            msg.setInformativeText("The format of the file must be : .SPE  .TIFF .sif or .txt ")
            msg.setWindowTitle("Warning ...")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec_()
            self.dataBgExist = False
    
    def checkBoxServerChange(self):
        
        if self. checkBoxServer.isChecked():
            # print('start number of shot Client')
            self.threadClient = THREADCLIENT(self)
            self.threadClient.start()
            self.threadClient.newShotnumber.connect(self.receiveNewNumber)
            self.threadClient.pathSignal.connect(self.receiveNewPath)
            self.threadClient.autoSignal.connect(self.receiveAuto)
        else:
            # print('close client')
            self.threadClient.stopClientThread()
    
    def receiveNewNumber(self, nbShot):
        self.tirNumberBox.setValue(nbShot)
        
    def receiveNewPath(self,path):
        self.pathBox.setText(path)

    def receiveAuto(self,auto):
        self.auto = auto
        if auto == 'True':
            auto = True
        else : 
            auto = False 
        
        self.parent.checkBoxAutoSave.setChecked(auto)  # on passe en auto save sur visu 
        self.parent.autoSaveColor()
        if self.parent.parent is not None:
            # quand on passe en mode auto save la camera si elle existe passe aussi en mode trig et en play
            try: 
                if auto == True:
                    self.parent.parent.trigg.setCurrentIndex(1)
                    # time.sleep(0.1)
                    self.parent.parent.runButton.click()
                    self.modeTrig = True
                else:
                    if self.modeTrig == True: # quand on decoche le mode autosave camera en mode free run 
                        #  on stop 1 fois mais apress on peut faire play sur la camera 
                        self.parent.parent.stopButton.click()
                        self.modeTrig = False
                    # self.parent.parent.trigg.setCurrentIndex(0)
            except Exception as e:
                print('error auto play or stop', e)

    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        self.closeEventVar.emit(True)


class THREADCLIENT(QtCore.QThread):
    '''
    Thread client ZMQ 
    Tous les événements (SHOOT, CONFIG) sont reçus via PUB/SUB
    '''
    newShotnumber = Signal(int)   # Signal pour le numéro de tir
    pathSignal = Signal(str)      # Signal pour le path
    autoSignal = Signal(str)      # Signal pour autosave
    
    def __init__(self, parent):
        super(THREADCLIENT, self).__init__(parent)
        self.parent = parent
        self.conf = self.parent.conf
        self.name = self.parent.name
        
        # Lire la configuration
        self.serverHost = str(self.conf.value(self.name + "/server"))
        self.serverPort = int(self.conf.value(self.name + "/serverPort"))#, "5009")) # ? ??
        
        self.ClientIsConnected = False
        self.client_id = str(uuid.uuid4())
        
        # envoyer heartbeat au serveur 
        self.last_heartbeat = time.time()
        self.heartbeat_interval = 10 # interval de temps pour envoyer les heartbeat au serveur 
        
        # ZMQ context et sockets
        self.context = None
        self.sub_socket = None
        self.pub_socket = None
        
    def run(self):

        print(f"Connecting to ZMQ server: {self.serverHost}:{self.serverPort}")
        
        try:
            # Créer le contexte ZMQ
            self.context = zmq.Context()
            
            # Socket SUB pour recevoir les événements du serveur
            self.sub_socket = self.context.socket(zmq.SUB)
            self.sub_socket.connect(f"tcp://{self.serverHost}:{self.serverPort}")
            
            # S'abonner à tous les types d'événements
            self.sub_socket.setsockopt_string(zmq.SUBSCRIBE, "SHOOT")      # Événements de tir
            self.sub_socket.setsockopt_string(zmq.SUBSCRIBE, "CONFIG")     # Mises à jour path/autosave
            self.sub_socket.setsockopt_string(zmq.SUBSCRIBE, "REGISTERED") # Confirmation d'enregistrement client
            self.sub_socket.setsockopt_string(zmq.SUBSCRIBE, "HEARTBEAT")  # Heartbeat serveur pour gerer si deconnection

            # Socket PUB pour envoyer notre enregistrement
            self.pub_socket = self.context.socket(zmq.PUB)
            self.pub_socket.connect(f"tcp://{self.serverHost}:{self.serverPort + 1}")
            
            # Petite pause pour que la connexion s'établisse
            time.sleep(0.1)
            
            # S'enregistrer auprès du serveur
            self._send_register()
            
            self.ClientIsConnected = True
            #print(f'Client connected to server {self.serverHost}')
            
            # Mettre à jour l'interface
            self.parent.nameBox.setText(self.parent.name)
            
        except Exception as e:
            print(f'Connection error: {e}')
            print('Do you start the server?')
            self.ClientIsConnected = False
            self.parent.checkBoxServer.setChecked(False)
            return
        
        # Poller pour gérer les timeouts
        poller = zmq.Poller()
        poller.register(self.sub_socket, zmq.POLLIN)
        
        # Variables pour détecter la déconnexion
        last_heartbeat = time.time()
        self.heartbeat_timeout = 15.0  # 10 secondes sans heartbeat = serveur déconnecté

        # Boucle principale - Écouter les événements
        while self.ClientIsConnected:
            try:
                # Attendre un événement avec timeout de 100ms
                socks = dict(poller.poll(100))
                
                if self.sub_socket in socks and socks[self.sub_socket] == zmq.POLLIN:
                    # Recevoir l'événement
                    topic = self.sub_socket.recv_string()
                    event = self.sub_socket.recv_json()
                    
                    # Dispatcher selon le type d'événement
                    if topic == "SHOOT":
                        #print('client : shoot nb received')
                        self._handle_shoot_event(event)
                        last_heartbeat = time.time()  # Reset heartbeat
                    elif topic == "CONFIG":
                        self._handle_config_event(event)
                        last_heartbeat = time.time()  # Reset heartbeat
                    elif topic == "REGISTERED":
                        self._handle_registered_event(event)
                        last_heartbeat = time.time()  # Reset heartbeat
                    elif topic == "HEARTBEAT":
                        last_heartbeat = time.time()
                        # print("💓 Heartbeat received")
                else:
                    # vérifier si serveur toujours vivant
                    time_since_heartbeat = time.time() - last_heartbeat
                    
                    if time_since_heartbeat > self.heartbeat_timeout:
                        print(f"❌ Server timeout ({time_since_heartbeat:.1f}s without response)")
                        self.ClientIsConnected = False
                        # Fermer les sockets
                        if self.sub_socket:
                            self.sub_socket.close()
                        if self.pub_socket:
                            self.pub_socket.close()
                        if self.context:
                            self.context.term()
                        self.parent.checkBoxServer.setChecked(False)
                        time.sleep(1)
                        break

                # Petite pause pour ne pas surcharger le CPU
                time.sleep(0.01)
                if time.time() - self.last_heartbeat > self.heartbeat_interval :
                    self.send_hearbeat()

            except zmq.ZMQError as e:
                if e.errno == zmq.ETERM:
                    break  # Context terminé
                #print(f'ZMQ Error: {e}')
                self.ClientIsConnected = False
                self.parent.checkBoxServer.setChecked(False)
                break
            except Exception as e:
                #print(f'Error in client loop: {e}')
                import traceback
                traceback.print_exc()
                self.ClientIsConnected = False
                self.parent.checkBoxServer.setChecked(False)
                break
        
        # Nettoyage
        self._cleanup()
    
    def _send_register(self):
        """Envoyer notre enregistrement au serveur"""
        register_event = {
            'client_id': self.client_id,
            'name': self.name,
            'timestamp': time.time()
        }
        
        # Publier sur le topic REGISTER
        self.pub_socket.send_string("REGISTER", zmq.SNDMORE)
        self.pub_socket.send_json(register_event)
        
       # print(f"Sent registration for {self.name}")
    
    def _handle_registered_event(self, event):
        """Gérer la confirmation d'enregistrement"""
        client_id = event.get('client_id')
        if client_id == self.client_id:
            # print(f"Registration confirmed by server")
            # Optionnel: traiter la config initiale si le serveur l'envoie
            if 'path' in event:
                path = event['path']
                if sys.platform == 'linux':
                    lpath = "/mnt/SJ_NAS/"
                    path = path.replace("X:/", lpath)
                self.pathSignal.emit(path)
            
            if 'autosave' in event:
                self.autoSignal.emit(str(event['autosave']))
    
    def _handle_shoot_event(self, event):
        """
        Gérer un événement de tir reçu
    
        """
        nbshot = event.get('number')
        timestamp = event.get('timestamp') # pour aline si besoin
         # print('clien shoot receveid,nbshot', nbshot)
        # Émettre le signal si le numéro a changé
        if int(self.parent.tirNumberBox.value()) != nbshot:
            self.newShotnumber.emit(nbshot)
        
    
    def _handle_config_event(self, event):
        """
        Gérer un événement de mise à jour de configuration
        Permet au serveur de pousser des changements de config en temps réel
        """
        # Vérifier si c'est pour nous
        client_id = event.get('client_id')
        if client_id and client_id != self.client_id:
            return  # Pas pour nous
        
        # Mise à jour du path
        if 'path' in event:
            path = event['path']
            
            # Adaptation Linux
            if sys.platform == 'linux':
                lpath = "/mnt/SJ_NAS/"      # pas tres propre mais bon ...
                path = path.replace("X:/", lpath)
            
            if str(self.parent.pathBox.text()) != path:
                self.pathSignal.emit(path)
                # print(f"Path updated to: {path}")
        
        # Mise à jour de l'autosave
        if 'autosave' in event:
            autosave = str(event['autosave'])
            # print('autosave received', autosave)
            if autosave != self.parent.auto:
                self.autoSignal.emit(autosave)
                # print(f"Autosave updated to: {autosave}")
    
    def _cleanup(self):
        """Nettoyer les ressources"""
        try:
            # Envoyer un événement de déconnexion
            unregister_event = {
                'client_id': self.client_id,
                'name': self.name
            }
            self.pub_socket.send_string("UNREGISTER", zmq.SNDMORE)
            self.pub_socket.send_json(unregister_event)
            self.parent.checkBoxServer.setChecked(False)
        except:
            pass
        
        # Fermer les sockets
        if self.sub_socket:
            self.sub_socket.close()
        if self.pub_socket:
            self.pub_socket.close()
        if self.context:
            self.context.term()
    
    def send_hearbeat(self):
        """fct pouur envoyer heartbeat au serveur """

        self.pub_socket.send_string("CLIENT_HEARTBEAT",zmq.SNDMORE)
        self.pub_socket.send_json({
            'client_id':self.client_id,
            'timestamp':time.time()
        })
        self.last_heartbeat = time.time()

    def stopClientThread(self):
        """Arrêter le thread client"""
        self.ClientIsConnected = False
        time.sleep(0.1)
        self.quit()
        self.wait()



if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = OPTION()
    e.show()
    appli.exec_() 


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Apr 12 21:39:01 2021

@author: juliengautier
"""


from PyQt6.QtWidgets import QApplication, QVBoxLayout, QWidget
from PyQt6.QtWidgets import QLabel
from PyQt6.QtGui import QIcon
from PyQt6.QtCore import Qt
import visu
import pathlib
import os
import sys
import qdarkstyle

__version__ = visu.__version__
__author__ = visu.__author__


class ABOUT(QWidget):
    
    def __init__(self, file=None, path=None, **kwds):
        
        super().__init__()
        self.version = __version__
        p = pathlib.Path(__file__)
        sepa = os.sep
        self.title = "About Visualization"
        self.icon = str(p.parent) + sepa+'icons' + sepa
        self.isWinOpen = False
        self.setWindowTitle(self.title)
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.setup()
        
    def setup(self):
        hMainLayout = QVBoxLayout()
        img = self.icon+'LOA.png'
        
        text1 = QLabel('Visualization  Version :  '+self.version)
        text1.setStyleSheet("font:22pt")
        text1.setAlignment(Qt.AlignmentFlag.AlignCenter)
        
        text2 = QLabel("<a href=\"https://loa.ensta-paris.fr/fr/accueil/\">Laboratoire d'optique Appliquée</a>")
        text2.setAlignment(Qt.AlignCenter)
        
        text2.setOpenExternalLinks(True)
        text2.setTextFormat(Qt.RichText)
        
        text3 = QLabel(__author__)
        text3.setAlignment(Qt.AlignCenter)
        text31 = QLabel("<a href=\"https://github.com/julienGautier77/visu/wiki/How--to-use-Visu\">visit on GitHub</a>")
        text31.setOpenExternalLinks(True)
        text31.setTextFormat(Qt.RichText)
        
        
        text4 = QLabel()
        text4.setStyleSheet("QLabel:!pressed{border-image: url(%s);background-color: transparent ;border-color: green;}""QLabel:pressed{image: url(%s);background-color: gray ;border-color: gray}" % (img, img))           
        text4.setMaximumWidth(60)
        text4.setMaximumHeight(60)
        text4.setAlignment(Qt.AlignCenter)
        
        hMainLayout.addWidget(text1)
        hMainLayout.addWidget(text2)
        hMainLayout.addWidget(text3)
        hMainLayout.addWidget(text31)
        hMainLayout.addWidget(text4)
        self.setLayout(hMainLayout)
   
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        event.accept()


if __name__ == "__main__":
    appli = QApplication(sys.argv) 
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = ABOUT()  
    e.show()
    appli.exec_()     



# -*- coding: utf-8 -*-

"""

Created on Tue Aug 13 15:27:31 2019
windows for see max ,mean ,sum in a bigger windows
@author: Salle-Jaune

"""

import qdarkstyle
from PyQt6.QtWidgets import QApplication, QHBoxLayout, QWidget, QLabel
from PyQt6.QtGui import QIcon
import sys
import time
import pathlib
import os


class ZOOM(QWidget):
    
    def __init__(self, symbol=True, title='Zoom'):
        super().__init__()
        p = pathlib.Path(__file__)
        sepa = os.sep
        self.title = title
        self.icon = str(p.parent) + sepa+'icons' + sepa
        self.isWinOpen = False
        self.symbol = symbol
        self.setup()
        self.setWindowTitle(self.title)
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    
    def setup(self):
        hLayout2 = QHBoxLayout()
        self.label = QLabel('?')
        self.label.setMinimumHeight(100)
        self.label.setMinimumWidth(400)
        self.label.setMaximumHeight(200)
        self.label.setMaximumWidth(500)
        self.label.setStyleSheet('font :bold  60pt;color: white')
        hLayout2.addWidget(self.label)
        self.setLayout(hLayout2)
        
    def setZoom(self, dat):
        if self.title == 'Sum':
            self.label.setText('%.3e' % dat)
        else:
            self.label.setText('%.0f' % dat)
        
    def SetTITLE(self, title):
        self.title = title
        self.setWindowTitle(title)
        
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        time.sleep(0.1)
        event.accept()
    

if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = ZOOM()
    e.show()
    appli.exec_()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec 19 11:43:05 2018
@author: juliengautier

"""

from PyQt6.QtWidgets import QApplication, QVBoxLayout, QHBoxLayout, QWidget, QGridLayout
from PyQt6.QtWidgets import QCheckBox, QLabel, QSizePolicy, QSpinBox
from PyQt6 import QtCore, QtGui
from PyQt6.QtCore import Qt
from PyQt6.QtGui import QShortcut
from PyQt6.QtGui import QIcon
import sys
import time
import pyqtgraph as pg  # pyqtgraph biblio permettent l'affichage
import numpy as np
import qdarkstyle  # pip install qdakstyle https://github.com/ColinDuquesnoy/QDarkStyleSheet  sur conda
import pylab
import os
from scipy.ndimage.filters import gaussian_filter  # pour la reduction du bruit
from scipy.interpolate import splrep, sproot  # pour calcul fwhm et fit
import pathlib


class WINENCERCLED(QWidget):

    def __init__(self, parent=None, conf=None, name='VISU'):
        
        super().__init__()

        self.name = name
        self.parent = parent
        p = pathlib.Path(__file__)

        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else:
            self.conf = conf
        
        sepa = os.sep
        self.icon = str(p.parent) + sepa+'icons' + sepa

        self.isWinOpen = False
        self.setWindowTitle('Encercled')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.left = 100
        self.top = 30
        self.width = 800
        self.height = 800
        self.setGeometry(self.left, self.top, self.width, self.height)
        self.dimx = 1200
        self.dimy = 900
        self.bloqq = 1
        self.xec = int(self.conf.value(self.name+"/xec"))
        self.yec = int(self.conf.value(self.name+"/yec"))
        self.r1x = int(self.conf.value(self.name+"/r1x"))
        self.r1y = int(self.conf.value(self.name+"/r1y"))
        self.r2 = int(self.conf.value(self.name+"/r2x"))
        self.r2 = int(self.conf.value(self.name+"/r2y"))
        
        self.kE = 0  # variable pour la courbe E fct du nb shoot
        
        self.Xec = []
        self.Yec = []
        self.fwhmX = 100
        self.fwhmY = 100
        self.setWindowIcon(QIcon('./icons/LOA.png'))
        self.E = []
        
        # self.E=np.array([2,3,5])
        self.Xec = []
        self.Yec = []
        
        # Create x and y indices
        x = np.arange(0, self.dimx)
        y = np.arange(0, self.dimy)
        y, x = np.meshgrid(y, x)
    
        self.data = (40*np.random.rand(self.dimx, self.dimy)).round()
        
        self.setup()
        self.ActionButton()
        self.Display(self.data)
        
    def setup(self):
        
        TogOff = self.icon+'Toggle_Off.png'
        TogOn = self.icon+'Toggle_On.png'       
        TogOff = pathlib.Path(TogOff)
        TogOff = pathlib.PurePosixPath(TogOff)
        TogOn = pathlib.Path(TogOn)
        TogOn = pathlib.PurePosixPath(TogOn)
        self.setStyleSheet("QCheckBox::indicator{width: 30px;height: 30px;}""QCheckBox::indicator:unchecked { image : url(%s);}""QCheckBox::indicator:checked { image:  url(%s);}""QCheckBox{font :10pt;}" % (TogOff, TogOn))
        
        vbox1 = QVBoxLayout()
        hbox = QHBoxLayout()
        self.checkBoxAuto = QCheckBox('Auto', self)
        self.checkBoxAuto.setChecked(True)
        hbox.addWidget(self.checkBoxAuto)
        self.bckButton = QCheckBox('Soustract Bg', self)
        hbox.addWidget(self.bckButton)
        vbox1.addLayout(hbox)
        hbox0 = QHBoxLayout()
        self.energieRes = QLabel('?')
        self.energieRes.setMaximumHeight(30)
        self.energieRes.setMaximumWidth(140)
        self.energieRes.setStyleSheet("color:blue;font:20pt")
        self.lEnergie = QLabel('s(E1)/s(E2) :')
        self.lEnergie.setStyleSheet("color:blue;font:18pt")
        self.lEnergie.setMaximumWidth(160)
        hbox0.addWidget(self.lEnergie)
        hbox0.addWidget(self.energieRes)
        # vbox1.addStretch(1)
        vbox1.addLayout(hbox0)
        
        LabelR1x = QLabel("fwhm X/0.85")
        LabelR1x.setStyleSheet("color:red;font:14pt")
        self.r1xBox = QSpinBox()
        # self.r1Box.setMaximumWidth(60)
        self.r1xBox.setMaximum(2000)
        LabelR1y = QLabel('fwhm Y/0.85')
        LabelR1y.setStyleSheet("color:green;font:14pt")
        self.r1yBox = QSpinBox()
        self.r1yBox.setMaximum(2000)
        
        LabelR2 = QLabel('R2')
        LabelR2.setStyleSheet("color:yellow;font:14pt")
        self.r2Box = QSpinBox()
        self.r2Box.setMaximum(2000)
        self.r2Box.setMaximumWidth(60)

        LabelE1 = QLabel("E1 Sum ")
        LabelE1.setStyleSheet("color:red;font:14pt")
        self.LabelE1Sum = QLabel("? ")
        self.LabelE1Sum.setStyleSheet("color:red;font:14pt")
        LabelE1M = QLabel("E1 mean ")
        LabelE1M.setStyleSheet("color:red;font:14pt")
        self.LabelE1Mean = QLabel("? ")
        self.LabelE1Mean.setStyleSheet("color:red;font:14pt")
        
        LabelE2 = QLabel("E2 Sum ")
        LabelE2.setStyleSheet("color:yellow;font:14pt")
        self.LabelE2Sum = QLabel("? ")
        self.LabelE2Sum.setStyleSheet("color:yellow;font:14pt")
        LabelE2M = QLabel("E2 mean ")
        LabelE2M.setStyleSheet("color:yellow ;font:14pt")
        self.LabelE2Mean = QLabel("? ")
        self.LabelE2Mean.setStyleSheet("color:yellow;font:14pt")
        
        grid_layout1 = QGridLayout()
        grid_layout1.addWidget(LabelR1x, 0, 0)
        grid_layout1.addWidget(self.r1xBox, 0, 1)
        grid_layout1.addWidget(LabelR1y, 1, 0)
        grid_layout1.addWidget(self.r1yBox, 1, 1)
        grid_layout1.addWidget(LabelR2, 2, 0)
        grid_layout1.addWidget(self.r2Box, 2, 1)
        grid_layout1.addWidget(LabelE1, 3, 0)
        grid_layout1.addWidget(self.LabelE1Sum, 3, 1)
        grid_layout1.addWidget(LabelE1M, 4, 0)
        grid_layout1.addWidget(self.LabelE1Mean, 4, 1)
        
        grid_layout1.addWidget(LabelE2, 5, 0)
        grid_layout1.addWidget(self.LabelE2Sum, 5, 1)
        grid_layout1.addWidget(LabelE2M, 6, 0)
        grid_layout1.addWidget(self.LabelE2Mean, 6, 1)
        
        vbox1.addLayout(grid_layout1)
        vbox1.addStretch(1)
        
        self.winImage = pg.GraphicsLayoutWidget()
        self.winImage.setContentsMargins(0, 0, 0, 0)
        self.winImage.setAspectLocked(True)
        self.winImage.setSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.MinimumExpanding)
        # self.winImage.ci.setContentsMargins(0,0,0,0)
        vbox2 = QVBoxLayout()
        hbox2 = QHBoxLayout()
        hbox2.addWidget(self.winImage)
        vbox2.addLayout(hbox2)
        vbox2.setContentsMargins(0, 0, 0, 0)
        
        self.p1 = self.winImage.addPlot()
        
        self.imh = pg.ImageItem()
        self.p1.addItem(self.imh)
        
        self.p1.setMouseEnabled(x=False, y=False)
        self.p1.setContentsMargins(0, 0, 0, 0)
        
        self.p1.setAspectLocked(True, ratio=1)
        self.p1.showAxis('right', show=False)
        self.p1.showAxis('top', show=False)
        self.p1.showAxis('left', show=False)
        self.p1.showAxis('bottom', show=False)
        self.vLine = pg.InfiniteLine(angle=90, movable=False, pen='w')
        self.hLine = pg.InfiniteLine(angle=0, movable=False, pen='w')
        self.p1.addItem(self.vLine)
        self.p1.addItem(self.hLine)
        
        self.vLine.setPos(self.xec)
        self.hLine.setPos(self.yec)
        
        self.roi1 = pg.CircleROI([self.xec, self.yec], [2*self.r1x, 2*self.r1y], pen='r', movable=False)
        self.roi1.setPos([self.xec-(self.r1x), self.yec-(self.r1y)])
        self.p1.addItem(self.roi1)
       
        self.roi2 = pg.CircleROI([self.xec, self.yec], [2*self.r2, 2*self.r2], pen='y', movable=False)
        self.roi2.setPos([self.xec-(self.r2), self.yec-(self.r2)])
        self.p1.addItem(self.roi2)
        
        # histogramme
        self.hist = pg.HistogramLUTItem() 
        self.hist.setImageItem(self.imh)
        self.hist.autoHistogramRange()
        self.hist.gradient.loadPreset('flame')
        
        
        self.curve2 = pg.PlotCurveItem()
        self.curve3 = pg.PlotCurveItem()
        # text pour afficher fwhm sur p1
        self.textX = pg.TextItem(angle=-90)
        self.textY = pg.TextItem()
        self.p1.addItem(self.curve2)
        self.p1.addItem(self.curve3)
        self.p1.addItem(self.textX)
        self.p1.addItem(self.textY)
        
        self.ROIRect = pg.RectROI([self.xec, self.yec], [4*self.r1x, 4*self.r1y], pen='m',)
        self.ROIRect.setPos([self.xec-(self.r1x), self.yec-(self.r1y)])
        hLayout1 = QHBoxLayout()
        hLayout1.addLayout(vbox2)
        hLayout1.addLayout(vbox1)
        hLayout1.setContentsMargins(1, 1, 1, 1)
        
        vMainLayout = QVBoxLayout()
        vMainLayout.addLayout(hLayout1)

        hMainLayout = QHBoxLayout()
        hMainLayout.addLayout(vMainLayout)
        self.setLayout(hMainLayout)
        self.setContentsMargins(1, 1, 1, 1)
        
    def ActionButton(self):
        # Blocage de la souris
        self.roi1.sigRegionChangeFinished.connect(self.energSouris)  # signal si changement à la souris de la taille des cercles
        self.roi2.sigRegionChangeFinished.connect(self.energSouris)
#
        self.r1xBox.editingFinished.connect(self.Rayon)  # w.r1Box.returnPressed.connect(Rayon)# rayon change a la main
        self.r1yBox.editingFinished.connect(self.Rayon)
        self.r2Box.editingFinished.connect(self.Rayon)
       
        self.shortcutb = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+b"), self)
        self.shortcutb.activated.connect(self.bloquer)
        self.shortcutd = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+d"), self)
        self.shortcutd.activated.connect(self.debloquer)
         
        self.shortcutPu = QShortcut(QtGui.QKeySequence("+"), self)
        self.shortcutPu.activated.connect(self.paletteup)
        self.shortcutPu.setContext(Qt.ShortcutContext(3))
        # 3: The shortcut is active when its parent widget, or any of its children has focus. default O The shortcut is active when its parent widget has focus.
        self.shortcutPd = QtGui.QShortcut(QtGui.QKeySequence("-"), self)
        self.shortcutPd.activated.connect(self.palettedown)
        self.shortcutPd.setContext(Qt.ShortcutContext(3))
        
        self.checkBoxAuto.stateChanged.connect(lambda: self.Display(self.data))
        # mvt de la souris
        self.proxy = pg.SignalProxy(self.p1.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.p1.scene().sigMouseClicked.connect(self.mouseClick)
        self.vb = self.p1.vb
        
        self.checkBoxAuto.stateChanged.connect(self.AutoE)
        self.bckButton.stateChanged.connect(self.Back)

        self.ROIRect.sigRegionChangeFinished.connect(self.roiBackChanged)
        
        if self.parent is not None:
            self.parent.signalEng.connect(self.Display)
        
    def mouseClick(self):
        # bloque ou debloque la souris si click su le graph
        if self.bloqq == 1:
            self.debloquer()
        else:
            self.bloquer() 
            
    def bloquer(self):  # bloque la croix 
        self.bloqq = 1
        self.CalculE()
        
    def debloquer(self):  # deblaoque la croix : elle bouge avec la souris
        self.bloqq = 0
    
    # mvt de la souris
    
    def mouseMoved(self, evt):
        # pour que la cross suive le mvt de la souris
        if self.checkBoxAuto.isChecked() is False:

            if self.bloqq == 0:  # souris non bloquer
                pos = evt[0]  # using signal proxy turns original arguments into a tuple
                if self.p1.sceneBoundingRect().contains(pos):
                    mousePoint = self.vb.mapSceneToView(pos)
                    self.xec = int(mousePoint.x())
                    self.yec = int(mousePoint.y())
                    if ((self.xec > 0 and self.xec < self.data.shape[0]) and (self.yec > 0 and self.yec < self.data.shape[1])):
                        self.vLine.setPos(self.xec)
                        self.hLine.setPos(self.yec)  # la croix ne bouge que dans le graph
                        self.roi1.setPos([self.xec-(self.r1x), self.yec-(self.r1y)])
                        self.roi2.setPos([self.xec-(self.r2), self.yec-(self.r2)])
                            
    def AutoE(self):
        if self.checkBoxAuto.isChecked() is False:
            self.energSouris()
            self.roi1.setSize([2*self.r1x, 2*self.r1y])
            self.roi2.setSize([2*self.r2, 2*self.r2])
            
    def energSouris(self):  # changement des rayons à la souris 
        if self.checkBoxAuto.isChecked() is False:
            s1 = self.roi1.size()
            s2 = self.roi2.size()
            self.r1xBox.setValue((int(s1[0]/2)))
            self.r1yBox.setValue((int(s1[1]/2)))
            self.r2Box.setValue((int(s2[0]/2)))
            self.r1x = int(s1[0]/2)
            self.r1y = int(s1[1]/2)
            self.r2 = int(s2[0]/2)
            if self.bloqq == 1:
                self.CalculE()
     
    def Rayon(self): 
        """changement rayon dans les box
        """
        if self.checkBoxAuto.isChecked() is False:
            if self.r1xBox.hasFocus():
                self.r1x = (float(self.r1xBox.text()))
                self.roi1.setSize([2*self.r1x, 2*self.r1y])
            if self.r1yBox.hasFocus():
                self.r1y = (float(self.r1yBox.text()))
                self.roi1.setSize([2*self.r1x, 2*self.r1y])
            
            if self.r2Box.hasFocus():
                self.r2 = (float(self.r2Box.text()))
                self.roi2.setSize([2*self.r2, 2*self.r2])
            
            self.roi1.setPos([self.xec-(self.r1x), self.yec-(self.r1y)])
            self.roi2.setPos([self.xec-(self.r2), self.yec-(self.r2)])
            if self.bloqq == 1:
                self.CalculE()
    
    def CalculE(self):
        
        if self.fwhmX is None or self.fwhmY is None:
            self.fwhmX = 100
            self.fwhmY = 100
        
        if self.checkBoxAuto.isChecked() is True:
            self.r1x = self.fwhmX/0.849
            self.r1y = self.fwhmY/0.849
            self.r1xBox.setValue(int(self.r1x))
            self.r1yBox.setValue(int(self.r1y))
            nbG = 2  # ré= 2 fois r1 pour le grand cercle
            self.roi2.setSize([2*nbG*self.r1x, 2*nbG*self.r1y])
            self.roi2.setPos([self.xec-nbG*self.r1x, self.yec-nbG*self.r1y])
            self.r2Box.setValue(int(2*nbG*self.r1x))
            self.roi1.setSize([2*self.r1x, 2*self.r1y])
            self.roi1.setPos([self.xec-self.r1x, self.yec-self.r1y])
            
        else:
            self.r1x = self.r1x
            self.r1y = self.r1y
           
        E1 = self.roi1.getArrayRegion(self.data, self.imh).sum()
        E2 = self.roi2.getArrayRegion(self.data, self.imh).sum()
        self.rap = 100*E1/E2
        self.energieRes.setText('%.2f %%' % self.rap)
        # self.E=np.append(self.E,self.rap)
        # #self.E.append(self.rap)
        # Emean=np.mean(self.E)
        # self.meanAff.setText('%.2f' % Emean)
        # EPV=np.std(self.E)
        # self.PVAff.setText('%.2f' % EPV)
        # self.hLineMeanE.setPos(Emean)
        self.textX.setText('fwhm='+str(self.fwhmY))
        self.textY.setText('fwhm='+str(self.fwhmX), color='w')
        self.LabelE1Sum.setText('%.2f' % E1)
        self.LabelE1Mean.setText('%.2f' % (self.roi1.getArrayRegion(self.data, self.imh).mean()))
        self.LabelE2Sum.setText('%.2f' % E2)
        self.LabelE2Mean.setText('%.2f' % (self.roi2.getArrayRegion(self.data, self.imh).mean()))
        
    def Display(self, data):
        self.dataOrg = data
        self.data = data
        
        self.dimx = self.data.shape[0]
        self.dimy = self.data.shape[1]
        self.p1.setXRange(0, self.dimx)
        self.p1.setYRange(0, self.dimy)
        self.p1.setAspectLocked(True, ratio=1)
        self.imh.setImage(data.astype(float), autoLevels=True, autoDownsample=True)
        self.CalculCentroid()
        self.Coupe()
        self.Back()
    
    def CalculCentroid(self):
        
        if self.checkBoxAuto.isChecked() is True:
            dataF = gaussian_filter(self.data, 5)
            (self.xec, self.yec) = pylab.unravel_index(dataF.argmax(), self.data.shape)  # prend le max 
            self.vLine.setPos(self.xec)
            self.hLine.setPos(self.yec)   
            self.roi1.setPos([self.xec-(self.r1x), self.yec-(self.r1y)])
            self.roi2.setPos([self.xec-(self.r2), self.yec-(self.r2)])
        
    def fwhm(self, x, y, order=3):
        
        """
            Determine full-with-half-maximum of a peaked set of points, x and y.
            Assumes that there is only one peak present in the datasset.  The function
            uses a spline interpolation of order k.
        """
        y = gaussian_filter(y, 5)  # filtre pour reduire le bruit
        half_max = np.amax(y)/2
        try:
            s = splrep(x, y - half_max, k=order)  # Find the B-spline representation of 1-D curve.
            roots = sproot(s)  # Given the knots (>=8) and coefficients of a cubic B-spline return the roots of the spline.
        except:
            roots = 0
           
        if len(roots) > 2:
            pass
            # print( "The dataset appears to have multiple peaks, and ","thus the FWHM can't be determined.")
        elif len(roots) < 2:
            pass
            #  print( "No proper peaks were found in the data set; likely ","the dataset is flat (e.g. all zeros).")
        else:
            return np.around(abs(roots[1] - roots[0]), decimals=2)
         
    def Coupe(self):
        
        xxx = np.arange(0, int(self.dimx), 1)
        yyy = np.arange(0, int(self.dimy), 1)
        coupeX = self.data[int(self.xec),:]
        coupeXMax = np.max(coupeX)
        
        if coupeXMax == 0:  # evite la div par zero
            coupeXMax = 1
            
        coupeXnorm = (self.data.shape[0]/10)*(coupeX/coupeXMax)  # normalise les coupes pour pas prendre tout l ecran
        self.curve2.setData(30+coupeXnorm, yyy, clear=True)
        
        coupeY = self.data[:, int(self.yec)]
        coupeYMax = np.max(coupeY)
        if coupeYMax == 0:
            coupeYMax = 1
            
        coupeYnorm = (self.data.shape[1]/10)*(coupeY/coupeYMax)
        self.curve3.setData(xxx, 20 + coupeYnorm, clear=True)
        
        # affichage de fwhm sur les coupes X et Y que si le max est >20 counts
        xCXmax = np.amax(coupeXnorm)  # max
        if xCXmax > 20:
            self.fwhmY = self.fwhm(yyy, coupeXnorm, order=3)
            
            yCXmax = yyy[coupeXnorm.argmax()]
            self.textX.setPos(xCXmax-3, yCXmax+3)
            
        yCYmax = np.amax(coupeYnorm)  # max
        if yCYmax > 20:
            self.fwhmX = self.fwhm(xxx, coupeYnorm, order=3)
            xCYmax = xxx[coupeYnorm.argmax()]            
            
            self.textY.setPos(xCYmax-3, yCYmax-3) 

    def paletteup(self):
        levels = self.imh.getLevels()
        if levels[0] is None:
            xmax = self.data.max()
            xmin = self.data.min()
        else:
            xmax = levels[1]
            xmin = levels[0]
            
        self.imh.setLevels([xmin, xmax + (xmax - xmin) / 10])
        # hist.setImageItem(imh,clear=True)
        self.hist.setHistogramRange(xmin, xmax)

    def palettedown(self):
        levels = self.imh.getLevels()
        if levels[0] is None:
            xmax = self.data.max()
            xmin = self.data.min()
        else:
            xmax = levels[1]
            xmin = levels[0]
            
        self.imh.setLevels([xmin, xmax - (xmax - xmin) / 10])
        # hist.setImageItem(imh,clear=True)
        self.hist.setHistogramRange(xmin, xmax)

    def roiBackChanged(self):
        bg = self.ROIRect.getArrayRegion(self.dataOrg, self.imh).mean()
        # print(bg)
        self.data = self.dataOrg-bg
        self.CalculE()
    
    def Back(self):
        if self.bckButton.isChecked() is True:
            self.p1.addItem(self.ROIRect)
            bg = self.ROIRect.getArrayRegion(self.dataOrg, self.imh).mean()
            self.data = self.dataOrg - bg
            self.CalculE()
        else:
            self.p1.removeItem(self.ROIRect)
            self.data = self.dataOrg
            self.CalculE()
              
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        self.E = []
        self.Xec = []
        self.Yec = []
        time.sleep(0.1)
        event.accept()
     
        
if __name__ == "__main__":
    appli = QApplication(sys.argv) 
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = WINENCERCLED(name='VISU')
    e.show()
    appli.exec_()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb  8 14:46:20 2019
Windows for plot
@author: juliengautier
"""

import pyqtgraph as pg
import qdarkstyle

from PyQt6 import QtCore, QtGui
from PyQt6.QtWidgets import QApplication, QHBoxLayout, QWidget, QStatusBar
from PyQt6.QtWidgets import QVBoxLayout, QLabel, QPushButton, QMessageBox
from PyQt6.QtWidgets import QColorDialog, QInputDialog, QGridLayout
from PyQt6.QtWidgets import QDoubleSpinBox, QMainWindow
from PyQt6.QtWidgets import QTableWidget, QTableWidgetItem, QFileDialog
from PyQt6.QtWidgets import QListWidget, QListWidgetItem
from PyQt6.QtGui import QAction
from PyQt6.QtGui import QIcon
from PyQt6.QtCore import Qt
import numpy as np
import pathlib
from scipy.optimize import curve_fit
from scipy.ndimage import gaussian_filter
from scipy.interpolate import splrep, sproot
import sys
import time
import os


class WINDOWRANGE(QWidget):
    """Small widget to set axis range"""
    def __init__(self):
        super().__init__()
        self.isWinOpen = False
        self.setup()
        
    def setup(self):
        hRangeGrid = QGridLayout()
        
        self.labelXmin = QLabel('Xmin:')
        self.xMinBox = QDoubleSpinBox(self)
        self.xMinBox.setMinimum(-100000)
        self.xMinBox.setMaximum(100000)
        hRangeGrid.addWidget(self.labelXmin, 0, 0)
        hRangeGrid.addWidget(self.xMinBox, 0, 1)
        self.labelXmax = QLabel('Xmax:')
        self.xMaxBox = QDoubleSpinBox(self)
        self.xMaxBox.setMaximum(100000)
        self.xMaxBox.setMinimum(-100000)
        hRangeGrid.addWidget(self.labelXmax, 1, 0)
        hRangeGrid.addWidget(self.xMaxBox, 1, 1)
        
        self.labelYmin = QLabel('Ymin:')
        self.yMinBox = QDoubleSpinBox(self)
        self.yMinBox.setMinimum(-100000)
        self.yMinBox.setMaximum(100000)
        hRangeGrid.addWidget(self.labelYmin, 2, 0)
        hRangeGrid.addWidget(self.yMinBox, 2, 1)
        self.labelYmax = QLabel('Ymax:')
        self.yMaxBox = QDoubleSpinBox(self)
        self.yMaxBox.setMaximum(100000)
        self.yMaxBox.setMinimum(-100000)
        hRangeGrid.addWidget(self.labelYmax, 3, 0)
        hRangeGrid.addWidget(self.yMaxBox, 3, 1)
        self.applyButton = QPushButton('Apply')
        self.ResetButton = QPushButton('Reset')
        hRangeGrid.addWidget(self.applyButton, 4, 0)
        hRangeGrid.addWidget(self.ResetButton, 4, 1)
        self.setLayout(hRangeGrid)

    def closeEvent(self, event):
        self.isWinOpen = False
        time.sleep(0.1)
        event.accept()


class PlotHistoryWidget(QWidget):
    """Widget pour gérer l'historique des plots"""
    def __init__(self, parent=None, max_plots=20):
        super().__init__()
        self.parent = parent
        self.isWinOpen = False
        self.max_plots = max_plots
        p = pathlib.Path(__file__)
        sepa = os.sep
        self.icon = str(p.parent) + sepa + 'icons' + sepa
        self.setWindowTitle('Plot History Manager')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.setGeometry(100, 100, 350, 500)
        self.setup()
        
    def setup(self):
        mainLayout = QVBoxLayout()
        mainLayout.setSpacing(15)
        mainLayout.setContentsMargins(15, 15, 15, 15)
        
        # ===== HEADER =====
        titleLabel = QLabel('📊 Plot History')
        titleLabel.setStyleSheet("""
            font: bold 16pt;
            color: #4dabf7;
            padding: 10px;
        """)
        titleLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)
        mainLayout.addWidget(titleLabel)
        
        # ===== PLOT LIST =====
        listLabel = QLabel('Select plots to display:')
        listLabel.setStyleSheet("font: bold 11pt; color: #adb5bd;")
        mainLayout.addWidget(listLabel)
        
        self.plotList = QListWidget()
        self.plotList.setSelectionMode(QListWidget.SelectionMode.MultiSelection)
        # Changer le signal - utiliser itemClicked au lieu de itemChanged
        self.plotList.itemChanged.connect(self.onItemChanged)
        self.plotList.setStyleSheet("""
            QListWidget {
                background-color: #1e1e1e;
                border: 2px solid #495057;
                border-radius: 8px;
                padding: 5px;
                font: 10pt;
            }
            QListWidget::item {
                padding: 10px;
                margin: 3px;
                border-radius: 5px;
            }
            QListWidget::item:hover {
                background-color: #343a40;
            }
            QListWidget::item:selected {
                background-color: #495057;
            }
        """)
        mainLayout.addWidget(self.plotList)
        
        # ===== ACTION BUTTONS =====
        buttonLayout = QHBoxLayout()
        buttonLayout.setSpacing(10)
        
        self.selectAllBtn = QPushButton('✓ Select All')
        self.selectAllBtn.setStyleSheet("""
            QPushButton {
                background-color: #4dabf7;
                color: white;
                border: none;
                border-radius: 5px;
                padding: 12px 20px;
                font: bold 10pt;
            }
            QPushButton:hover {
                background-color: #339af0;
            }
            QPushButton:pressed {
                background-color: #1c7ed6;
            }
        """)
        self.selectAllBtn.clicked.connect(self.selectAll)
        buttonLayout.addWidget(self.selectAllBtn)
        
        self.deselectAllBtn = QPushButton('✗ Deselect All')
        self.deselectAllBtn.setStyleSheet("""
            QPushButton {
                background-color: #495057;
                color: white;
                border: none;
                border-radius: 5px;
                padding: 12px 20px;
                font: bold 10pt;
            }
            QPushButton:hover {
                background-color: #343a40;
            }
            QPushButton:pressed {
                background-color: #212529;
            }
        """)
        self.deselectAllBtn.clicked.connect(self.deselectAll)
        buttonLayout.addWidget(self.deselectAllBtn)
        
        mainLayout.addLayout(buttonLayout)
        
        # ===== CLEAR BUTTON =====
        self.clearBtn = QPushButton('🗑️ Clear History')
        self.clearBtn.setStyleSheet("""
            QPushButton {
                background-color: #ff6b6b;
                color: white;
                border: none;
                border-radius: 5px;
                padding: 12px;
                font: bold 11pt;
            }
            QPushButton:hover {
                background-color: #fa5252;
            }
            QPushButton:pressed {
                background-color: #e03131;
            }
        """)
        self.clearBtn.clicked.connect(self.clearHistory)
        mainLayout.addWidget(self.clearBtn)
        
        # ===== INFO LABEL =====
        self.infoLabel = QLabel('')
        self.infoLabel.setStyleSheet("""
            color: #adb5bd;
            font: 10pt;
            padding: 8px;
            background-color: #2b2b2b;
            border-radius: 5px;
        """)
        self.infoLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)
        mainLayout.addWidget(self.infoLabel)
        
        # ===== WARNING LABEL =====
        self.warningLabel = QLabel('')
        self.warningLabel.setStyleSheet("""
            color: #ffd43b;
            font: bold 9pt;
            padding: 5px;
        """)
        self.warningLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.warningLabel.setVisible(False)
        mainLayout.addWidget(self.warningLabel)
        
        mainLayout.addStretch()
        
        self.setLayout(mainLayout)
        self.updateInfo()
    
    def addPlot(self, plot_id, label):
        """Ajouter un plot à la liste"""
        # Vérifier si la limite est atteinte
        if self.plotList.count() >= self.max_plots:
            # Supprimer le plus ancien (premier de la liste)
            self.removeOldestPlot()
            self.showWarning(f'⚠️ Maximum {self.max_plots} plots - oldest removed')
        
        # Créer l'item avec icône colorée
        item = QListWidgetItem()
        
        # Extraire la couleur du plot_id depuis le parent
        color = '#ffffff'
        if self.parent is not None:
            for plot_data in self.parent.plot_history:
                if plot_data['id'] == plot_id:
                    color = plot_data['color']
                    break
        
        # Formater le label avec une puce colorée
        formatted_label = f"●  {label}"
        item.setText(formatted_label)
        item.setForeground(QtGui.QColor(color))
        
        item.setFlags(item.flags() | Qt.ItemFlag.ItemIsUserCheckable)
        item.setCheckState(Qt.CheckState.Checked)
        item.setData(Qt.ItemDataRole.UserRole, plot_id)
        
        self.plotList.addItem(item)
        self.updateInfo()
    
    def removeOldestPlot(self):
        """Supprimer le plot le plus ancien"""
        if self.plotList.count() > 0:
            first_item = self.plotList.item(0)
            plot_id = first_item.data(Qt.ItemDataRole.UserRole)
            
            # Informer le parent de supprimer ce plot
            if self.parent is not None:
                self.parent.removePlotFromHistory(plot_id)
            
            self.plotList.takeItem(0)
    
    def onItemChanged(self, item):
        """Quand l'état de la checkbox change"""
        # Pas besoin de toggle - Qt gère ça automatiquement
        if self.parent is not None:
            plot_id = item.data(Qt.ItemDataRole.UserRole)
            is_visible = item.checkState() == Qt.CheckState.Checked
            self.parent.setPlotVisibility(plot_id, is_visible)
        
        self.updateInfo()
    
    def selectAll(self):
        """Sélectionner tous les plots"""
        for i in range(self.plotList.count()):
            item = self.plotList.item(i)
            if item.checkState() != Qt.CheckState.Checked:
                item.setCheckState(Qt.CheckState.Checked)
                # Mettre à jour la visibilité
                if self.parent is not None:
                    plot_id = item.data(Qt.ItemDataRole.UserRole)
                    self.parent.setPlotVisibility(plot_id, True)
        self.updateInfo()
    
    def deselectAll(self):
        """Désélectionner tous les plots"""
        for i in range(self.plotList.count()):
            item = self.plotList.item(i)
            if item.checkState() != Qt.CheckState.Unchecked:
                item.setCheckState(Qt.CheckState.Unchecked)
                # Mettre à jour la visibilité
                if self.parent is not None:
                    plot_id = item.data(Qt.ItemDataRole.UserRole)
                    self.parent.setPlotVisibility(plot_id, False)
        self.updateInfo()
    
    def clearHistory(self):
        """Effacer l'historique"""
        if self.plotList.count() == 0:
            return
        
        reply = QMessageBox.question(
            self, 
            'Clear History', 
            f'Are you sure you want to clear all {self.plotList.count()} plots from history?',
            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
        )
        if reply == QMessageBox.StandardButton.Yes:
            self.plotList.clear()
            if self.parent is not None:
                self.parent.clearPlotHistory()
            self.updateInfo()
            self.hideWarning()
    
    def updateInfo(self):
        """Mettre à jour les informations"""
        count = self.plotList.count()
        visible_count = sum(1 for i in range(count) if self.plotList.item(i).checkState() == Qt.CheckState.Checked)
        
        self.infoLabel.setText(f"Plots: {count}/{self.max_plots}  |  Visible: {visible_count}/{count}")  # max_plots au lieu de max_points
    
    def showWarning(self, message):
        """Afficher un message d'avertissement"""
        self.warningLabel.setText(message)
        self.warningLabel.setVisible(True)
        
        # Auto-hide après 3 secondes
        from PyQt6.QtCore import QTimer
        QTimer.singleShot(3000, self.hideWarning)
    
    def hideWarning(self):
        """Cacher le message d'avertissement"""
        self.warningLabel.setVisible(False)
    
    def closeEvent(self, event):
        self.isWinOpen = False
        event.accept()


class WINDOWMEAS(QWidget):
    def __init__(self, title='Plot Measurement'):
        
        super().__init__()
        self.isWinOpen = False
        p = pathlib.Path(__file__)
        sepa = os.sep
        self.title = title
        self.icon = str(p.parent) + sepa + 'icons' + sepa
        self.setWindowTitle(self.title)
        self.setWindowIcon(QIcon(self.icon + 'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.setup()
        self.setGeometry(50, 100, 200, 350)
    
    def setup(self):
        hLayout1 = QHBoxLayout()
        self.table = QTableWidget()
        hLayout1.addWidget(self.table)
        
        self.table.setColumnCount(1)
        self.table.setRowCount(7)
        self.table.setVerticalHeaderLabels(('Max', 'Min', 'x max', 'x min', 'Mean', 'PV', 'Std'))
        self.setLayout(hLayout1)
    
    def Display(self, cutData, axis=None, fwhm=False, axisOn=False, fit=False, **kwds):
       
        cutData = np.array(cutData)
        Max = round(max(cutData), 3)
        Min = round(min(cutData), 3)
        Mean = round(np.mean(cutData), 3)
        PV = Max - Min
        Std = round(np.std(cutData), 3)
        
        if axisOn is False:
            xmax = np.argmax(cutData)
            xmin = np.argmin(cutData)
        else:
            xmax = np.argmax(cutData)
            xmax = axis[xmax]
            xmin = np.argmin(cutData)
            xmin = axis[xmin]
        
        fit = fit
        self.table.setItem(0, 0, QTableWidgetItem(str(Max)))
        self.table.setItem(1, 0, QTableWidgetItem(str(Min)))
        self.table.setItem(2, 0, QTableWidgetItem(str(xmax)))
        self.table.setItem(3, 0, QTableWidgetItem(str(xmin)))
        self.table.setItem(4, 0, QTableWidgetItem(str(Mean)))
        self.table.setItem(5, 0, QTableWidgetItem(str(PV)))
        self.table.setItem(6, 0, QTableWidgetItem(str(Std)))
        
        if fit is True:
            fitA = kwds["fitA"]
            fitMu = kwds["fitMu"]
            fitSigma = kwds["fitSigma"]
            if fwhm is True:
                self.table.setRowCount(11)
                self.table.setVerticalHeaderLabels(('Max', 'Min', 'x max', 'x min', 'Mean', 'PV', 'Std', 'FWHM', 'Fit A', 'Fit Mu', 'Fit Sigma'))
                self.table.setItem(8, 0, QTableWidgetItem(str(fitA)))
                self.table.setItem(9, 0, QTableWidgetItem(str(fitMu)))
                self.table.setItem(10, 0, QTableWidgetItem(str(fitSigma)))
                
                if axisOn is False:
                    xxx = np.arange(0, np.shape(cutData)[0])
                else:
                    xxx = axis
                
                try:
                    fwhmValue = self.fwhm(xxx, cutData)[0]
                except Exception as e:
                    fwhmValue = ''
                self.table.setItem(7, 0, QTableWidgetItem(str(fwhmValue)))
                
            else:
                self.table.setRowCount(10)
                self.table.setVerticalHeaderLabels(('Max', 'Min', 'x max', 'x min', 'Mean', 'PV', 'Std', 'Fit A', 'Fit Mu', 'Fit Sigma'))
                self.table.setItem(7, 0, QTableWidgetItem(str(round(fitA, 3))))
                self.table.setItem(8, 0, QTableWidgetItem(str(round(fitMu, 3))))
                self.table.setItem(9, 0, QTableWidgetItem(str(round(fitSigma, 3))))
        else:
            if fwhm is True:
                self.table.setRowCount(8)
                self.table.setVerticalHeaderLabels(('Max', 'Min', 'x max', 'x min', 'Mean', 'PV', 'Std', 'FWHM'))
                if axisOn is False:
                    xxx = np.arange(0, np.shape(cutData)[0])
                else:
                    xxx = axis
                
                try:
                    fwhmValue = self.fwhm(xxx, cutData)[0]
                except:
                    fwhmValue = ''
                self.table.setItem(7, 0, QTableWidgetItem(str(round(fwhmValue, 3))))
            else:
                self.table.setVerticalHeaderLabels(('Max', 'Min', 'x max', 'x min', 'Mean', 'PV', 'Std'))
                self.table.setRowCount(7)
                  
    def fwhm(self, x, y, order=3):
        y = gaussian_filter(y, 5)
        half_max = np.amax(y)/2
        
        try:
            s = splrep(x, y - half_max, k=order)
            roots = sproot(s)
        except:
            roots = 0
        
        if len(roots) > 2:
            pass
        elif len(roots) < 2:
            pass
        else:
            return np.around(abs(roots[1] - roots[0]), decimals=2), half_max
    
    def closeEvent(self, event):
        self.isWinOpen = False
        time.sleep(0.1)
        event.accept()


class GRAPHCUT(QMainWindow):
    
    def __init__(self, parent=None, symbol=None, title='Plot', conf=None, name='VISU', meas=False, pen='w', symbolPen='w', label=None, labelY=None, clearPlot=True, lastColored=False):
        
        super().__init__()
        p = pathlib.Path(__file__)
        sepa = os.sep
        self.parent = parent
        self.title = title
        self.icon = str(p.parent) + sepa + 'icons' + sepa
        self.isWinOpen = False
        self.dimx = 10
        self.bloqq = 0
        self.xc = 0
        self.measWidget = WINDOWMEAS()
        self.meas = meas
        self.cutData = np.zeros(self.dimx)
        self.path = None
        self.axisOn = False
        self.lastColored = lastColored
        
        # Historique des plots
        self.plot_history = []  # Liste de dictionnaires contenant les données et paramètres
        self.plot_items = []  # Liste des objets PlotDataItem affichés
        self.plot_counter = 0  # Compteur pour générer des IDs uniques
        self.historyWidget = PlotHistoryWidget(parent=self)
        
        # Palette de couleurs pour les différents plots
        self.color_palette = [
            '#ff6b6b',  # Rouge
            '#51cf66',  # Vert
            '#4dabf7',  # Bleu
            '#ffd43b',  # Jaune
            '#a78bfa',  # Violet
            '#ff9f43',  # Orange
            '#5f3dc4',  # Violet foncé
            '#20c997',  # Turquoise
        ]
        
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else:
            self.conf = conf

        self.name = name
        
        self.setWindowTitle(self.title)
        self.setWindowIcon(QIcon(self.icon + 'LOA.png'))
        self.scatter = False
        self.symbol = symbol
        self.axis = None
        self.label = label
        self.labelY = labelY
        self.symbolPen = symbolPen
        self.symbolBrush = 'w'
        self.ligneWidth = 1
        self.color = 'w'
        self.plotRectZoomEtat = "Zoom"
        self.pen = pen
        self.clearPlot = clearPlot
        self.xLog = False
        self.yLog = False
        self.fitA = ""
        self.fitMu = ""
        self.fitSigma = ""
        self.widgetRange = WINDOWRANGE()
        self.setup()
        self.actionButton()
        
    def setup(self):
        
        self.toolBar = self.addToolBar('tools')
        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        self.fileMenu = menubar.addMenu('&File')
        self.ImageMenu = menubar.addMenu('&Plot option')
        self.axisMenu = menubar.addMenu('&Axis option')
        self.AnalyseMenu = menubar.addMenu('&Analyse')
        self.historyMenu = menubar.addMenu('&History')  # Nouveau menu History
        
        self.Aboutenu = menubar.addMenu('&About')
        self.statusBar = QStatusBar()
        self.setContentsMargins(5, 5, 5, 5)
        
        self.setStatusBar(self.statusBar)
        
        self.openAct = QAction(QtGui.QIcon(self.icon+"Open.png"), 'Open File', self)
        self.openAct.setShortcut('Ctrl+o')
        self.openAct.triggered.connect(self.OpenF)
        self.toolBar.addAction(self.openAct)
        self.fileMenu.addAction(self.openAct)
        
        self.saveAct = QAction(QtGui.QIcon(self.icon+"disketteSave.png"), 'Save file', self)
        self.saveAct.setShortcut('Ctrl+s')
        self.saveAct.triggered.connect(self.SaveF)
        self.toolBar.addAction(self.saveAct)
        self.fileMenu.addAction(self.saveAct)
        
        # Menu History
        self.showHistoryAct = QAction('Show History', self)
        self.showHistoryAct.setShortcut('Ctrl+h')
        self.showHistoryAct.triggered.connect(lambda: self.open_widget(self.historyWidget))
        self.historyMenu.addAction(self.showHistoryAct)
        
        self.clearHistoryAct = QAction('Clear All Plots', self)
        self.clearHistoryAct.triggered.connect(self.clearPlotHistory)
        self.historyMenu.addAction(self.clearHistoryAct)
        
        self.keepPlotsAct = QAction('Keep Plots in History', self)
        self.keepPlotsAct.setCheckable(True)
        self.keepPlotsAct.setChecked(False)
        self.historyMenu.addAction(self.keepPlotsAct)
        
        vLayout = QVBoxLayout()
        hLayout1 = QHBoxLayout()
        self.checkBoxPlot = QAction(QtGui.QIcon(self.icon+"target.png"), 'Cross On (ctrl+b to block ctrl+d to deblock)', self)
        self.checkBoxPlot.setCheckable(True)
        self.checkBoxPlot.setChecked(False)
        self.checkBoxPlot.triggered.connect(self.PlotXY)
        self.toolBar.addAction(self.checkBoxPlot)
        self.AnalyseMenu.addAction(self.checkBoxPlot)
        
        self.measAction = QAction('Measure')
        self.AnalyseMenu.addAction(self.measAction)
        self.measAction.triggered.connect(lambda: self.open_widget(self.measWidget))
        self.measAction.triggered.connect(lambda: self.measWidget.Display(cutData=self.cutData, axis=self.axis, axisOn=self.axisOn))
        
        self.fwhmAction = QAction('fwhm')
        self.fwhmAction.setCheckable(True)
        self.fwhmAction.setChecked(False)
        self.fwhmAction.triggered.connect(lambda: self.open_widget(self.measWidget))
        self.fwhmAction.triggered.connect(lambda: self.measWidget.Display(cutData=self.cutData, axis=self.axis, axisOn=self.axisOn, fwhm=self.fwhmAction.isChecked(), fit=self.fit, fitA=self.fitA, fitMu=self.fitMu, fitSigma=self.fitSigma))
       
        self.AnalyseMenu.addAction(self.fwhmAction)
        
        self.actionLigne = QAction('Set line', self)
        self.actionLigne.triggered.connect(self.setLine)
        self.actionLigne.setCheckable(True)
        if self.pen is not None:
            self.actionLigne.setChecked(True)
            
        self.ImageMenu.addAction(self.actionLigne)
        
        self.actionColor = QAction('Set line color', self)
        self.actionColor.triggered.connect(self.setColorLine)
        self.ImageMenu.addAction(self.actionColor)
        
        self.actionLigneWidth = QAction('Set line width', self)
        self.actionLigneWidth.triggered.connect(self.setWidthLine)
        self.ImageMenu.addAction(self.actionLigneWidth)
        
        self.label_CrossValue = QLabel()
        self.label_CrossValue.setStyleSheet("font:13pt")
        
        self.label_Cross = QLabel()
        self.label_Cross.setMaximumWidth(170)
        self.label_Cross.setStyleSheet("font:12pt")
        self.statusBar.addPermanentWidget(self.label_Cross)
        self.statusBar.addPermanentWidget(self.label_CrossValue)
        
        if self.meas is True:
            self.label_Mean = QLabel('Mean :')
            hLayout1.addWidget(self.label_Mean)
            self.label_MeanValue = QLabel('...')
            hLayout1.addWidget(self.label_MeanValue)
            self.label_PV = QLabel('PV :')
            hLayout1.addWidget(self.label_PV)
            self.label_PVValue = QLabel('...')
            hLayout1.addWidget(self.label_PVValue)
            self.label_Variance = QLabel('variance :')
            hLayout1.addWidget(self.label_Variance)
            self.label_VarianceValue = QLabel('...')
            hLayout1.addWidget(self.label_VarianceValue)
        
        self.checkBoxSymbol = QAction('Set Symbol on', self)
        self.checkBoxSymbol.setCheckable(True)
        
        if self.symbol is not None:
            self.checkBoxSymbol.setChecked(True)
        self.ImageMenu.addAction(self.checkBoxSymbol)
        self.checkBoxSymbol.triggered.connect(self.setSymbol)
        
        self.checkBoxSymbolColor = QAction('Set Symbol color', self)
        self.ImageMenu.addAction(self.checkBoxSymbolColor)
        self.checkBoxSymbolColor.triggered.connect(self.setColorSymbol)
        
        self.showGridX = QAction('Show X grid', self)
        self.showGridX.setCheckable(True)
        self.ImageMenu.addAction(self.showGridX)
        self.showGridX.triggered.connect(self.showGrid)
        
        self.showGridY = QAction('Show Y grid', self)
        self.showGridY.setCheckable(True)
        self.ImageMenu.addAction(self.showGridY)
        self.showGridY.triggered.connect(self.showGrid)
        
        self.lockGraphAction = QAction('Lock Graph', self)
        self.lockGraphAction.setCheckable(True)
        self.ImageMenu.addAction(self.lockGraphAction)
        self.lockGraphAction.triggered.connect(self.lockGraph)
        
        self.axisRange = QAction('Set Axis Range', self)
        self.axisMenu.addAction(self.axisRange)
        self.axisRange.triggered.connect(self.showRange)
        
        self.logActionX = QAction('Log X', self)
        self.axisMenu.addAction(self.logActionX)
        self.logActionX.setCheckable(True)
        self.logActionX.triggered.connect(self.logMode)
        
        self.logActionY = QAction('Log Y', self)
        self.axisMenu.addAction(self.logActionY)
        self.logActionY.setCheckable(True)
        self.logActionY.triggered.connect(self.logMode)
        
        self.ZoomRectButton = QAction(QtGui.QIcon(self.icon+"loupe.png"), 'Zoom', self)
        self.ZoomRectButton.triggered.connect(self.zoomRectAct)
        self.toolBar.addAction(self.ZoomRectButton)
        
        self.plotRectZoom = pg.RectROI([0, 0], [100, 100], pen='b')
            
        self.fitAction = QAction('Gaussian Fit', self)
        self.AnalyseMenu.addAction(self.fitAction)
        self.fitAction.setCheckable(True)
        
        self.fitAction.triggered.connect(self.setFit)
        self.fitAction.triggered.connect(lambda: self.open_widget(self.measWidget))
        self.fitAction.triggered.connect(lambda: self.measWidget.Display(cutData=self.cutData, axis=self.axis, axisOn=self.axisOn, fwhm=self.fwhmAction.isChecked(), fit=self.fit, fitA=self.fitA, fitMu=self.fitMu, fitSigma=self.fitSigma))
        
        self.vLine = pg.InfiniteLine(angle=90, movable=False, pen='y')
        self.hLine = pg.InfiniteLine(angle=0, movable=False, pen='y')
    
        self.vLine.setPos(0)
        self.hLine.setPos(0)
        
        vLayout.addLayout(hLayout1)
        hLayout2 = QHBoxLayout()
        
        self.winImage = pg.GraphicsLayoutWidget()
        self.winPLOT = self.winImage.addPlot()
        
        if self.label is not None:
            self.winPLOT.setLabel('bottom', self.label)
        if self.labelY is not None:
            self.winPLOT.setLabel('left', self.labelY)
        hLayout2.addWidget(self.winImage)
        vLayout.addLayout(hLayout2)
     
        MainWidget = QWidget()
        
        MainWidget.setLayout(vLayout)
        
        self.setCentralWidget(MainWidget)
        
        self.pCut = self.winPLOT.plot(symbol=self.symbol, symbolPen=self.symbolPen, symbolBrush=self.symbolBrush, pen=self.pen, clear=self.clearPlot)
        self.pCut2 = self.winPLOT.plot(symbol='o', symbolPen='g')
        
        pen = pg.mkPen(color='r', width=3)
        self.pFit = self.winPLOT.plot(pen=pen)

    def addLegend(self):
        self.winPLOT.addLegend(offset=(300, 100))

    def actionButton(self):
        self.proxy = pg.SignalProxy(self.winPLOT.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.vb = self.winPLOT.vb
        self.winPLOT.scene().sigMouseClicked.connect(self.mouseClick)
        self.widgetRange.applyButton.clicked.connect(self.setRangeOn)
        self.widgetRange.ResetButton.clicked.connect(self.setRangeReset)
        if self.parent is not None:
            self.parent.signalPlot.connect(self.PLOTSIG)
    
    def addToHistory(self, cutData, axis=None, label=None):
        """Ajouter un plot à l'historique"""
        if not self.keepPlotsAct.isChecked():
            return
        
        self.plot_counter += 1
        plot_id = self.plot_counter
        
        # Choisir une couleur dans la palette
        color_index = (self.plot_counter - 1) % len(self.color_palette)
        color = self.color_palette[color_index]
        
        # Créer un label pour l'historique
        if label is None:
            timestamp = time.strftime("%H:%M:%S")
            hist_label = f"Plot #{plot_id} - {timestamp}"
        else:
            hist_label = f"{label} (#{plot_id})"
        
        # Stocker les données
        plot_data = {
            'id': plot_id,
            'data': np.array(cutData),
            'axis': np.array(axis) if axis is not None else None,
            'label': hist_label,
            'color': color,
            'visible': True,
            'plot_item': None
        }
        
        self.plot_history.append(plot_data)
        
        # Créer le plot item
        pen = pg.mkPen(color=color, width=1.5)
        if axis is not None:
            plot_item = self.winPLOT.plot(y=cutData, x=axis, pen=pen, name=hist_label)
        else:
            plot_item = self.winPLOT.plot(cutData, pen=pen, name=hist_label)
        
        plot_data['plot_item'] = plot_item
        
        # Ajouter à la liste du widget historique
        self.historyWidget.addPlot(plot_id, hist_label)
        
        #  print(f"Added to history: {hist_label}")

    def removePlotFromHistory(self, plot_id):

        """Supprimer un plot spécifique de l'historique"""
        for i, plot_data in enumerate(self.plot_history):
            if plot_data['id'] == plot_id:
                # Supprimer du graphique
                if plot_data['plot_item'] is not None:
                    self.winPLOT.removeItem(plot_data['plot_item'])
                # Supprimer de la liste
                self.plot_history.pop(i)
                #  print(f"Removed plot #{plot_id} from history")
                break

    def setPlotVisibility(self, plot_id, visible):
        """Afficher/masquer un plot de l'historique"""
        for plot_data in self.plot_history:
            if plot_data['id'] == plot_id:
                plot_data['visible'] = visible
                if plot_data['plot_item'] is not None:
                    if visible:
                        plot_data['plot_item'].show()
                    else:
                        plot_data['plot_item'].hide()
                break
    
    def clearPlotHistory(self):
        """Effacer tout l'historique des plots"""
        # Supprimer tous les plot items du graphique
        for plot_data in self.plot_history:
            if plot_data['plot_item'] is not None:
                self.winPLOT.removeItem(plot_data['plot_item'])
        
        # Vider la liste
        self.plot_history.clear()
        self.plot_counter = 0
        
        # Réafficher le plot actuel
        self.CHANGEPLOT(self.cutData)
        
    def OpenF(self, fileOpen=False):

        if fileOpen is False:
            chemin = self.conf.value(self.name+"/path")
            fname = QFileDialog.getOpenFileName(self, "Open File", chemin, " 1D data (*.txt )")
            fichier = fname[0]
        else:
            fichier = str(fileOpen)
    
        ext = os.path.splitext(fichier)[1]
        
        if ext == '.txt':
            self.cutData = np.genfromtxt(fichier, delimiter=" ")
            self.PLOT(self.cutData)
        else:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Critical)
            msg.setText("Wrong file format !")
            msg.setInformativeText("The format of the file must be : .txt ")
            msg.setWindowTitle("Warning ...")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec()

    def SaveF(self):
        
        fname = QFileDialog.getSaveFileName(self, "Save data as txt", self.path)
        self.path = os.path.dirname(str(fname[0]))
        fichier = fname[0]
        
        print(fichier, ' is saved')
        self.conf.setValue(self.name+"/path", self.path)
        time.sleep(0.1)
        if self.axis is None:
            np.savetxt(str(fichier)+'.txt', self.cutData)
        else:
            saveData = np.array([self.axis, self.cutData])
            saveData = saveData.T
            np.savetxt(str(fichier)+'.txt', saveData)
        
    def mouseMoved(self, evt):
        
        if self.checkBoxPlot.isChecked() is False:
            
            if self.bloqq == 0:
                pos = evt[0]
                if self.winPLOT.sceneBoundingRect().contains(pos):
                
                    mousePoint = self.vb.mapSceneToView(pos)
                    
                    self.xMouse = (mousePoint.x())
                    self.yMouse = (mousePoint.y())
                    if self.axisOn is True:
                        
                        if (self.xMouse > self.axis.min()-10 and self.xMouse < self.axis.max() + 10):
                            self.xMc = self.xMouse
                            self.yMc = self.yMouse
                            self.label_Cross.setText('x=' + str(round((self.xMc), 2)) + ' y=' + str(round(self.yMc, 2)))
                    else:
                        if (self.xMouse > -1 and self.xMouse < self.dimx - 1):
                            self.xMc = int(self.xMouse)
                            self.yMc = self.cutData[self.xMc]
                            self.label_Cross.setText('x=' + str(round((self.xMc), 2)) + ' y=' + str(round(self.cutData[self.xMc], 2)))
                            
        if self.checkBoxPlot.isChecked() and self.bloqq == 0:
            pos = evt[0]
            if self.winPLOT.sceneBoundingRect().contains(pos):
                mousePoint = self.vb.mapSceneToView(pos)
                self.xMouse = (mousePoint.x())
                self.yMouse = (mousePoint.y())
                if self.axisOn is True:
                
                    if (self.xMouse > self.axis.min()-10 and self.xMouse < self.axis.max() + 10):
                        self.xc = int(self.xMouse)
                        self.yc = self.yMouse
                        self.vLine.setPos(self.xc)
                        self.hLine.setPos(self.yc)
                        self.affiCross()
                else: 
                    if (self.xMouse > -1 and self.xMouse < self.dimx - 1):
                        self.xc = int(self.xMouse)
                        self.yc = self.cutData[self.xc]
                        self.vLine.setPos(self.xc)
                        self.hLine.setPos(self.yc)     
                        self.affiCross()
    
    def affiCross(self):
        if self.checkBoxPlot.isChecked():
            if self.axisOn is True:
                self.label_Cross.setText('x=' + str(round((self.xc), 2)) + ' y=' + str(round(self.yc, 2)))
            else:
                self.label_Cross.setText('x=' + str(round((self.xc), 2)) + ' y=' + str(round(self.cutData[self.xc], 2)))
            
    def mouseClick(self):
        if self.bloqq == 1:
            self.bloqq = 0
        else:
            self.bloqq = 1
            
    def PlotXY(self):
        if self.checkBoxPlot.isChecked():
            self.winPLOT.addItem(self.vLine, ignoreBounds=False)
            self.winPLOT.addItem(self.hLine, ignoreBounds=False)
            self.vLine.setPos(self.xc)
            self.hLine.setPos(self.cutData[self.xc])
        else:
            self.winPLOT.removeItem(self.vLine)
            self.winPLOT.removeItem(self.hLine)
    
    def PLOTSIG(self, P):
        
        cutData = P['data']
        try:
            axis = P['axis']
        except:
            axis = None
        try:
            label = P['label']
        except:
            label = None
        try:
            labelY = P['labelY']
        except:
            labelY = None
                
        self.PLOT(cutData, axis=axis, label=label, labelY=labelY)
    
    def PLOT(self, cutData, axis=None, label=None, labelY=None):
        self.label = label
        self.labelY = labelY
        self.cutData = cutData
        self.axis = axis
        
        # Ajouter à l'historique si activé
        if self.keepPlotsAct.isChecked():
            self.addToHistory(cutData, axis=axis, label=label)
        
        self.dimy = max(cutData)
        self.minY = min(cutData)
        if self.axis is None:
            self.dimx = np.shape(self.cutData)[0]
            self.minX = 0
            self.data = self.cutData
            
            if self.clearPlot is False:
                self.pCut = self.winPLOT.plot(self.cutData, clear=self.clearPlot, symbol=self.symbol, symbolPen=self.symbolPen, symbolBrush=self.symbolBrush, pen=self.pen)          
            else:
                self.pCut.setData(self.data)
            self.axisOn = False
        else:
            self.axis = np.array(axis)
            self.dimx = max(self.axis)
            self.minX = min(self.axis)
            if self.clearPlot is False:
                self.pCut = self.winPLOT.plot(y=self.cutData, x=self.axis, clear=self.clearPlot, symbol=self.symbol, symbolPen=self.symbolPen, symbolBrush=self.symbolBrush, pen=self.pen)
            else:
                if self.lastColored is True:
                    
                    if self.scatter is True: 
                        self.winPLOT.removeItem(self.scatter_point)
                        
                    self.scatter_point = pg.ScatterPlotItem(
                        x=self.axis[-1].flatten(), 
                        y=self.cutData[-1].flatten(), 
                        pen=None,
                        symbol='d',
                        brush=pg.mkBrush(255, 0, 0),
                        size=15,
                        symbolBrush='r',
                        name='Last Shoot')
                    self.winPLOT.addItem(self.scatter_point)
                    self.scatter = True
                    self.uniqueAxis = np.unique(self.axis)
                    self.cutData = np.array(self.cutData)
                    self.axis = np.array(self.axis)
                    self.moy = np.array([np.mean(self.cutData[self.axis == val]) for val in self.uniqueAxis])
                    
                    self.pCut2.setData(y=self.moy, x=self.uniqueAxis, clear=False, symbol='o', symbolPen=self.symbolPen, symbolBrush='g', pen=self.pen, name='Mean')
                self.pCut.setData(y=self.cutData[:-1], x=self.axis[:-1], clear=self.clearPlot, symbol=self.symbol, symbolPen=self.symbolPen, symbolBrush=self.symbolBrush, pen=self.pen)
    
            self.axisOn = True
            
        self.zoomRectupdate()
        self.setFit()
        if self.label is not None:
            self.winPLOT.setLabel('bottom', self.label)
        if self.labelY is not None:
            self.winPLOT.setLabel('left', self.labelY)
        self.fit = self.fitAction.isChecked()
        
        self.measWidget.Display(cutData=self.cutData, axis=self.axis, axisOn=self.axisOn, fwhm=self.fwhmAction.isChecked(), fit=self.fit, fitA=self.fitA, fitMu=self.fitMu, fitSigma=self.fitSigma)
       
    def CHANGEPLOT(self, cutData):
        if self.axis is None:
            self.pCut = self.winPLOT.plot(self.cutData, clear=self.clearPlot, symbol=self.symbol, symbolPen=self.symbolPen, symbolBrush=self.symbolBrush, pen=self.pen)
        else:
            self.axisOn = True
            self.pCut = self.winPLOT.plot(y=self.cutData, x=self.axis, clear=self.clearPlot, symbol=self.symbol, symbolPen=self.symbolPen, symbolBrush=self.symbolBrush, pen=self.pen)
            
        if self.label is not None:
            self.winPLOT.setLabel('bottom', self.label)
        if self.labelY is not None:
            self.winPLOT.setLabel('left', self.labelY)
            
        self.affiCross()
        self.fit = self.fitAction.isChecked()

        if self.fitAction.isChecked():
            self.pFit = self.winPLOT.plot(pen='r')
            self.setFit()
            
        if self.meas is True:
            self.label_MeanValue.setText(str(round(np.mean(self.cutData), 3)))
            self.label_PVValue.setText(str(round(np.ptp(self.cutData), 3)))
            self.label_VarianceValue.setText(str(round(np.var(self.cutData), 3)))
       
    def SetTITLE(self, title):
        self.setWindowTitle(title)
    
    def dragEnterEvent(self, e):
        e.accept()

    def dropEvent(self, e):
        listFile = []
        for url in e.mimeData().urls():
            listFile.append(str(url.toLocalFile()))
        e.accept()
        self.OpenF(fileOpen=listFile[0])
    
    def setLine(self):
        if self.actionLigne.isChecked():
            self.pen = None
        else:
            self.pen = pg.mkPen(color=self.color, width=self.ligneWidth)
            
        self.CHANGEPLOT(self.cutData)
    
    def setColorLine(self):
        color = QColorDialog.getColor()
        self.color = str(color.name())
        self.pen = pg.mkPen(color=self.color, width=self.ligneWidth)
        self.CHANGEPLOT(self.cutData)
    
    def setWidthLine(self):
        num, ok = QInputDialog.getInt(self, "Line width", "enter a width ")
        if ok:
            self.ligneWidth = float(num)
            self.pen = pg.mkPen(color=self.color, width=self.ligneWidth)
            self.CHANGEPLOT(self.cutData)
            
    def setColorSymbol(self):
        color = QColorDialog.getColor()
        self.colorSymbol = str(color.name())
        self.symbolPen = pg.mkPen({'color': self.colorSymbol})
        self.symbolBrush = pg.mkBrush(self.colorSymbol)
        self.CHANGEPLOT(self.cutData)
    
    def showGrid(self):
        self.winPLOT.showGrid(x=self.showGridX.isChecked(), y=self.showGridY.isChecked())
        
    def setSymbol(self):
        if self.checkBoxSymbol.isChecked():
            self.symbol = 't'
        else:
            self.symbol = None
        
        self.CHANGEPLOT(self.cutData)
       
    def zoomRectAct(self):
        
        if self.plotRectZoomEtat == "Zoom":
            [[xmin, xmax], [ymin, ymax]] = self.winPLOT.viewRange()
            self.winPLOT.addItem(self.plotRectZoom)
            
            self.plotRectZoom.setPos([xmin+(xmax-xmin)/2, ymin+(ymax-ymin)/2])
            self.ZoomRectButton.setIcon(QtGui.QIcon(self.icon+"zoom-in.png"))
            self.plotRectZoomEtat = "ZoomIn"
            
        elif self.plotRectZoomEtat == "ZoomIn":
            self.ZoomRectButton.setIcon(QtGui.QIcon(self.icon+"zoom-out.png"))
            self.xZoomMin = (self.plotRectZoom.pos()[0])
            self.yZoomMin = (self.plotRectZoom.pos()[1])
            self.xZoomMax = (self.plotRectZoom.pos()[0])+self.plotRectZoom.size()[0]
            self.yZoomMax = (self.plotRectZoom.pos()[1])+self.plotRectZoom.size()[1]
            self.winPLOT.setXRange(self.xZoomMin, self.xZoomMax)
            self.winPLOT.setYRange(self.yZoomMin, self.yZoomMax)
            self.winPLOT.removeItem(self.plotRectZoom)
            
            self.plotRectZoomEtat = "ZoomOut"
        
        elif self.plotRectZoomEtat == "ZoomOut":
            self.winPLOT.setYRange(self.minY, self.dimy)
            self.winPLOT.setXRange(self.minX, self.dimx)
            self.ZoomRectButton.setIcon(QtGui.QIcon(self.icon+"loupe.png"))
            self.plotRectZoomEtat = "Zoom"
            
    def zoomRectupdate(self):
        if self.plotRectZoomEtat == "ZoomOut":
            self.winPLOT.setXRange(self.xZoomMin, self.xZoomMax)
            self.winPLOT.setYRange(self.yZoomMin, self.yZoomMax)
    
    def showRange(self):
        self.open_widget(self.widgetRange)
        
    def setRangeOn(self):
        self.xZoomMin = (self.widgetRange.xMinBox.value())
        self.yZoomMin = (self.widgetRange.yMinBox.value())
        self.xZoomMax = (self.widgetRange.xMaxBox.value())
        self.yZoomMax = (self.widgetRange.yMaxBox.value())
        self.winPLOT.setXRange(self.xZoomMin, self.xZoomMax)
        self.winPLOT.setYRange(self.yZoomMin, self.yZoomMax)
        self.plotRectZoomEtat = "ZoomIn"
        
    def setRangeReset(self):
        self.winPLOT.setYRange(self.minY, self.dimy)
        self.winPLOT.setXRange(self.minX, self.dimx)
        self.plotRectZoomEtat = "Zoom"
     
    def lockGraph(self):
        if self.lockGraphAction.isChecked():
            self.clearPlot = False
        else:
            self.clearPlot = True
        self.CHANGEPLOT(self.cutData)
    
    def logMode(self):
        if self.logActionY.isChecked():
            self.yLog = True
        else:
            self.yLog = False
        if self.logActionX.isChecked():
            self.xLog = True
        else:
            self.xLog = False
        self.winPLOT.setLogMode(x=self.xLog, y=self.yLog)
        self.CHANGEPLOT(self.cutData)
        
    def setXlog(self):
        self.logActionX.setChecked(True)
        self.xLog = True
        self.winPLOT.setLogMode(x=self.xLog, y=self.yLog)

    def setYlog(self):
        self.logActionY.setChecked(True)
        self.yLog = True
        self.winPLOT.setLogMode(x=self.xLog, y=self.yLog)

    def setFit(self):
        self.fit = self.fitAction.isChecked()
        if self.fitAction.isChecked():
            try:
                if self.axis is None:
                    xxx = np.arange(0, int(self.dimx), 1)
                else:
                    xxx = self.axis
            except:
                if self.axis.any is None:
                    xxx = np.arange(0, int(self.dimx), 1)
                else:
                    xxx = self.axis
            try:
                Datafwhm, xDataMax = self.fwhm(xxx, self.cutData)
                xmaxx = self.cutData.max()
                ymaxx = self.cutData[int(xmaxx)]
            except:
                xmaxx, ymaxx, Datafwhm = 0, 0, 0
              
            init_vals = [ymaxx, xmaxx, Datafwhm, 0]
            try:
                best_vals, covar = curve_fit(self.gauss, xxx, gaussian_filter(self.cutData, 5), p0=init_vals)
                y_fit = self.gauss(xxx, best_vals[0], best_vals[1], best_vals[2], best_vals[3])
                self.pFit.setData(x=xxx, y=y_fit)
            except:
                y_fit = [0]
                self.pFit.setData(x=[], y=[], clear=True)
            
            self.fitA = best_vals[0]
            self.fitMu = best_vals[1]
            self.fitSigma = best_vals[2]
            
        else:
            self.pFit.setData(x=[], y=[], clear=True)
            
    def open_widget(self, fene):
        if fene.isWinOpen is False:
            fene.setup
            fene.isWinOpen = True
            fene.show()
        else:
            fene.raise_()
            fene.showNormal()
    
    def fwhm(self, x, y, order=3):
        y = gaussian_filter(y, 5)
        half_max = np.amax(y)/2
        
        try:
            s = splrep(x, y - half_max, k=order)
            roots = sproot(s)
        except:
            roots = 0
        
        if len(roots) > 2:
            pass
        elif len(roots) < 2:
            pass
        else:
            return np.around(abs(roots[1] - roots[0]), decimals=2), half_max
    
    def gauss(self, x, A, mu, sigma, B):
        if sigma == 0:
            return 0
        else:
            return A*np.exp(-(x-mu)**2/(2.*sigma**2))+B
    
    def plot2D(self, cutData, axis=None, symbol=None, pen='r', symbolPen='w', clearPlot=False, legend=''):
        self.symbol = symbol
        self.symbolPen = symbolPen
        self.pen = pen
        self.cutData = cutData
        self.axis = axis
        self.dimy = max(cutData)
        self.minY = min(cutData)
        if self.axis is None:
            self.dimx = np.shape(self.cutData)[0]
            self.minX = 0
            self.data = self.cutData
            self.pCut = self.winPLOT.plot(y=cutData, clear=clearPlot, symbol=symbol, symbolPen=symbolPen, pen=pen, name=legend)
            self.axisOn = False
        else:
            self.axis = np.array(axis)
            self.dimx = max(self.axis)
            self.minX = min(self.axis)
            self.pCut = self.winPLOT.plot(y=cutData, x=axis, clear=clearPlot, symbol=symbol, symbolPen=symbolPen, pen=pen, name=legend)
            self.axisOn = True

    def closeEvent(self, event):
        self.isWinOpen = False
        time.sleep(0.1)
        event.accept()
        if self.measWidget.isWinOpen is True:
            self.measWidget.close()
        if self.widgetRange.isWinOpen is True:
            self.widgetRange.close()
        if self.historyWidget.isWinOpen is True:
            self.historyWidget.close()


if __name__ == "__main__":

    appli = QApplication(sys.argv)
    e = GRAPHCUT()
    a = [2, 3, 7, 100, 1000]
    b = [2, 4, 5, 100, 2000]
    e.PLOT(a, b)
    e.show()
    appli.exec()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 10 09:44:07 2019
Window for Measurement
@author: juliengautier
modified 2019/08/13 : add motors RSAI position and zoom windows
"""


import qdarkstyle
from PyQt6 import QtCore
from PyQt6.QtWidgets import QApplication, QVBoxLayout, QHBoxLayout, QMainWindow
from PyQt6.QtWidgets import QWidget, QTableWidget, QTableWidgetItem, QAbstractItemView
import sys
import os
from PyQt6.QtGui import QIcon
import pathlib


class HISTORY(QMainWindow):
    
    def __init__(self, parent=None, conf=None, name='VISU', confMot=None, **kwds):
        
        super().__init__()

        self.parent = parent
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        p = pathlib.Path(__file__)
        sepa = os.sep
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else:
            self.conf = conf
        self.confMot = confMot   
        self.name = name
        self.shoot = 0
        self.nbHistory = 20  # maximum file to  be displayed in history log 
        self.isWinOpen = False
        self.setup()
        self.setWindowTitle('History')
        self.nomFichier = ''
        self.path = self.conf.value(self.name+"/path")
        self.labelsVert = []
        self.icon = str(p.parent) + sepa+'icons' + sepa
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        
    def setup(self):
        
        vLayout = QVBoxLayout()
        hLayout2 = QHBoxLayout()
        self.table = QTableWidget()
        hLayout2.addWidget(self.table)
        
        self.table.setColumnCount(1)
        # self.table.setRowCount(10)
        self.table.horizontalHeader().setVisible(True)
        self.table.setHorizontalHeaderLabels(('File', ''))
        self.table.setAlternatingRowColors(True)
        self.table.resizeColumnsToContents()
        self.table.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)  # no modifiable
        
        vLayout.addLayout(hLayout2)
        MainWidget = QWidget()
        MainWidget.setLayout(vLayout)
        self.setCentralWidget(MainWidget)
        self.setContentsMargins(1, 1, 1, 1)
        self.table.cellDoubleClicked.connect(self.openF)
       
    def Display(self, file):
        
        self.table.selectRow(self.shoot)

        if self.shoot >= self.nbHistory:
            for i in range(0, self.shoot+1):
                
                item = self.table.item(i+1, 0)
                
                self.table.setItem(i, 0, QTableWidgetItem(item))
             
            self.table.setItem(self.shoot-1, 0, QTableWidgetItem(str(file)))
            print('shoot', self.shoot)
        else:
            self.table.setRowCount(self.shoot+1)
            self.table.setVerticalHeaderLabels(self.labelsVert)
            self.table.setItem(self.shoot, 0, QTableWidgetItem(str(file)))
            self.shoot += 1
            
        self.table.resizeColumnsToContents()
        
    def openF(self, row, column):
        item = self.table.item(row, column)
        if self.parent is not None:
            print('parent', self.parent)
            self.parent.OpenF(fileOpen=item.text())
        
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False


if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = HISTORY()
    e.show()
    appli.exec_()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 18 14:06:58 2019
open('README.md').read()
'PyQt6-sip== 13.5.2','PyQt6-qt6==6.5.1','PyQt6==6.5.1',
install_requires=['matplotlib','PyQt6-sip== 13.5.2','PyQt6-qt6==6.5.1','PyQt6==6.5.1','qdarkstyle','pyqtgraph','scipy','six','zmq',])
@author: juliengautier
"""

import re
from pathlib import Path
from setuptools import setup, find_packages

This_directory = Path(__file__).parent
long_description = (This_directory / "README.md").read_text()


def get_version():
    """Lit la version depuis visu/__init__.py"""
    init_file = This_directory / "visu" / "__init__.py"
    content = init_file.read_text(encoding='utf-8')
    match = re.search(r"^__version__\s*=\s*['\"]([^'\"]+)['\"]", content, re.MULTILINE)
    if match:
        return match.group(1)
    raise RuntimeError("Version non trouvée dans visu/__init__.py")


setup(
    name='visu',
    version=get_version(),
    packages=find_packages(),
    author='Julien Gautier',
    author_email='julien.gautier@ensta.fr',
    description='Data visualization',
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    package_data={'visu': ['icons/*.*']},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Free for non-commercial use",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Visualization"
    ],
    url='https://github.com/julienGautier77/visu',
    install_requires=[
        'matplotlib',
        'PyQt6',
        'qdarkstyle',
        'pyqtgraph',
        'scipy',
        'six',
        'pyzmq',
    ],
)
